@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -26,14 +26,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
 //-------------------------------------------------------------------------------
-DialogLoadGRIB::DialogLoadGRIB() : QDialog()
+DialogLoadGRIB::DialogLoadGRIB()
 {
     loadgrib = new FileLoaderGRIB();
     assert(loadgrib);
 
     oldcursor = cursor();
     
-    setWindowTitle(tr("Téléchargement - GRIB"));
+    setWindowTitle(tr("Download - GRIB"));
     loadInProgress = false;
     QFrame * frameButtonsZone = createFrameButtonsZone(this);
 
@@ -99,8 +99,16 @@ DialogLoadGRIB::DialogLoadGRIB() : QDialog()
     connect(chkTempMin, SIGNAL(stateChanged(int)), 	this,  SLOT(slotParameterUpdated()));
     connect(chkTempMax, SIGNAL(stateChanged(int)), 	this,  SLOT(slotParameterUpdated()));
     connect(chkSnowCateg, SIGNAL(stateChanged(int)), 	this,  SLOT(slotParameterUpdated()));
+    connect(chkFrzRainCateg, SIGNAL(stateChanged(int)), 	this,  SLOT(slotParameterUpdated()));
     connect(chkSnowDepth, SIGNAL(stateChanged(int)), 	this,  SLOT(slotParameterUpdated()));
     connect(chkCAPEsfc, SIGNAL(stateChanged(int)), 	this,  SLOT(slotParameterUpdated()));
+	
+    connect(chkAltitude200, SIGNAL(stateChanged(int)), 	this,  SLOT(slotParameterUpdated()));
+    connect(chkAltitude300, SIGNAL(stateChanged(int)), 	this,  SLOT(slotParameterUpdated()));
+    connect(chkAltitude500, SIGNAL(stateChanged(int)), 	this,  SLOT(slotParameterUpdated()));
+    connect(chkAltitude700, SIGNAL(stateChanged(int)), 	this,  SLOT(slotParameterUpdated()));
+    connect(chkAltitude850, SIGNAL(stateChanged(int)), 	this,  SLOT(slotParameterUpdated()));
+    connect(chkAltitudeAll, SIGNAL(stateChanged(int)), 	this,  SLOT(slotAltitudeAll()));
 }
 
 //-------------------------------------------------------------------------------
@@ -128,8 +136,8 @@ void DialogLoadGRIB::slotGribDataReceived(QByteArray *content, QString fileName)
 		path += "/";
 	// version >= 2.2.0 : file is bziped
 	fileName = fileName + ".bz2";
-    fileName = QFileDialog::getSaveFileName(this,
-                 tr("Sauvegarde du fichier GRIB"), path+fileName, "");
+    fileName = QFileDialog::getSaveFileName (this,
+                 tr("Save GRIB file"), path+fileName, "");
     if (fileName != "")
     {
         QFile saveFile (fileName);
@@ -146,23 +154,23 @@ void DialogLoadGRIB::slotGribDataReceived(QByteArray *content, QString fileName)
         }
         if (ok && nb>0) {
             loadInProgress = false;
-            btCancel->setText(tr("Annuler"));
+            btCancel->setText(tr("Cancel"));
             btOK->setEnabled(true);
             accept();
             emit signalMeteoFileReceived(fileName);
         }
         else {
             QMessageBox::critical (this,
-                    tr("Erreur"),
-                    tr("Ecriture du fichier impossible."));
+                    tr("Error"),
+                    tr("Can't write file."));
         }
     }
     else {
-        QMessageBox::critical (this,
-                tr("Erreur"),
-                tr("Opération abandonnée."));
+/*        QMessageBox::critical (this,
+                tr("Error"),
+                tr("Operation canceled."));*/
         loadInProgress = false;
-        btCancel->setText(tr("Annuler"));
+        btCancel->setText(tr("Cancel"));
         btOK->setEnabled(true);
         slotGribReadProgress(1,0,100);
     }
@@ -174,14 +182,14 @@ void DialogLoadGRIB::slotGribFileError(QString error)
 	setCursor(oldcursor);
     if (! loadInProgress)
         return;
-    
+ //DBGQS(error);   
     QString s;
     QMessageBox::critical (this,
-                    tr("Erreur"),
-                    tr("Erreur : ") + error );
+                    tr("Error"),
+                    tr("Error: ") + error );
     
     loadInProgress = false;
-    btCancel->setText(tr("Annuler"));
+    btCancel->setText(tr("Cancel"));
     btOK->setEnabled(true);
     labelMsg->setText("");
 }
@@ -202,18 +210,20 @@ void DialogLoadGRIB::slotGribReadProgress(int step, int done, int total)
     else {
         progressBar->setRange(0,total);
         progressBar->setValue(done);
-/****
-        // temps estimé
-        int elapsed = timeLoad.elapsed();
-        QTime duree(0,0);
-        duree = duree.addMSecs(elapsed);
-        QString eta = duree.toString("HH:mm:ss");
-     ****/
-        
-        slotGribMessage(tr("Taille : %1 ko    Reçus : %2 ko ")
-                .arg( total/1024 )
-                .arg( done/1024 )
-        );
+
+		QString speedunit = tr("ko/s");
+		double speed = done/1024/(timeLoad.elapsed()/1000.0);
+		if (speed > 1024) {
+			speed = speed/1024;
+			speedunit = tr("Mo/s");
+		}
+
+		slotGribMessage(tr("Size: %1 ko      Done: %2 ko at %3 %4")
+                .arg( total/1024, 5)
+                .arg( done/1024, 5)
+				.arg(speed,0,'f',1)
+				.arg(speedunit)
+			);
     }
 }
 
@@ -290,6 +300,12 @@ void DialogLoadGRIB::updateParameters()
 	Util::setSetting("downloadSnowCateg", snowCateg);
 	Util::setSetting("downloadFrzRainCateg", frzRainCateg);
 	Util::setSetting("downloadCAPEsfc", CAPEsfc);
+	
+	Util::setSetting("downloadAltitudeData200",  chkAltitude200->isChecked());
+	Util::setSetting("downloadAltitudeData300",  chkAltitude300->isChecked());
+	Util::setSetting("downloadAltitudeData500",  chkAltitude500->isChecked());
+	Util::setSetting("downloadAltitudeData700",  chkAltitude700->isChecked());
+	Util::setSetting("downloadAltitudeData850",  chkAltitude850->isChecked());
 }
 
 //-------------------------------------------------------------------------------
@@ -318,38 +334,45 @@ void DialogLoadGRIB::slotParameterUpdated()
     int nbFrzRainCateg = frzRainCateg ?  nbrec-1  : 0;
     int nbCAPEsfc  = CAPEsfc ?  nbrec : 0;
     
-    
-//    int estime = (nbWind+nbPress+nbRain+nbCloud+nbTemp) * (nbx*nby*2+84);
     int head = 84;
     int estime = 0;
     int nbits;
     
+    nbits = 13;
+    estime += nbWind*(head+(nbits*npts)/8+2 );
     nbits = 11;
     estime += nbTemp*(head+(nbits*npts)/8+2 );
     estime += nbTempPot*(head+(nbits*npts)/8+2 );
     estime += nbTempMin*(head+(nbits*npts)/8+2 );
     estime += nbTempMax*(head+(nbits*npts)/8+2 );
-    estime += nbSnowDepth*(head+(nbits*npts)/8+2 );
-    nbits = 12;
+    nbits = 4;
     estime += nbRain*(head+(nbits*npts)/8+2 );
-    nbits = 13;
-    estime += nbWind*(head+(nbits*npts)/8+2 );
-    nbits = 16;
+    nbits = 15;
     estime += nbPress*(head+(nbits*npts)/8+2 );
-    nbits = 8;
+    nbits = 4;
     estime += nbCloud*(head+(nbits*npts)/8+2 );
-    nbits = 7;
+    nbits = 1;
+    estime += nbSnowDepth*(head+(nbits*npts)/8+2 );
     estime += nbSnowCateg*(head+(nbits*npts)/8+2 );
     estime += nbFrzRainCateg*(head+(nbits*npts)/8+2 );
     nbits = 10;
     estime += nbHumid*(head+(nbits*npts)/8+2 );
-    estime += nbCAPEsfc*(head+(nbits*npts)/8+2 );
-    nbits = 16;
+    nbits = 15;
     estime += nbIsotherm0*(head+(nbits*npts)/8+2 );
+    nbits = 5;
+    estime += nbCAPEsfc*(head+(nbits*npts)/8+2 );
 
+	int nbalt = 0;
+	if (chkAltitude200->isChecked()) nbalt++;
+	if (chkAltitude300->isChecked()) nbalt++;
+	if (chkAltitude500->isChecked()) nbalt++;
+	if (chkAltitude700->isChecked()) nbalt++;
+	if (chkAltitude850->isChecked()) nbalt++;
+	nbits = 12;
+	estime += nbrec*nbalt*5*(head+(nbits*npts)/8+2 );
+	
     estime = estime/1024;
-    
-    slotGribMessage(tr("Taille estimée : environ %1 ko").arg(estime) );
+    slotGribMessage(tr("Size: %1 ko approx").arg(estime) );
     
     if (estime == 0)
         btOK->setEnabled(false);
@@ -358,6 +381,16 @@ void DialogLoadGRIB::slotParameterUpdated()
 
 }
 
+//-------------------------------------------------------------------------------
+void DialogLoadGRIB::slotAltitudeAll ()
+{
+	bool check = chkAltitudeAll->isChecked ();
+	chkAltitude200->setChecked (check);
+	chkAltitude300->setChecked (check);
+	chkAltitude500->setChecked (check);
+	chkAltitude700->setChecked (check);
+	chkAltitude850->setChecked (check);
+}
 //-------------------------------------------------------------------------------
 void DialogLoadGRIB::slotBtOK()
 {
@@ -372,7 +405,12 @@ void DialogLoadGRIB::slotBtOK()
 					resolution, interval, days,
 					wind, pressure, rain, cloud, temp, humid, isotherm0,
 					tempPot, tempMin, tempMax, snowDepth, snowCateg, frzRainCateg,
-					CAPEsfc
+					CAPEsfc, 
+					chkAltitude200->isChecked(),
+					chkAltitude300->isChecked(),
+					chkAltitude500->isChecked(),
+					chkAltitude700->isChecked(),
+					chkAltitude850->isChecked()
 				);
 }
 //-------------------------------------------------------------------------------
@@ -391,7 +429,7 @@ void DialogLoadGRIB::slotBtCancel()
     {
         loadInProgress = false;
         loadgrib->stop();
-        btCancel->setText(tr("Annuler"));
+        btCancel->setText(tr("Cancel"));
         progressBar->setRange(0,100);
         progressBar->setValue(0);
         slotParameterUpdated();
@@ -416,14 +454,15 @@ void DialogLoadGRIB::setZone(float x0, float y0, float x1, float y1)
 //-------------------------------------------------------------------------------
 QFrame *DialogLoadGRIB::createFrameButtonsZone(QWidget *parent)
 {
+    QFrame * ftmp;
     QFrame * frm = new QFrame(parent);
     assert(frm);
     QVBoxLayout  *lay = new QVBoxLayout(frm);
     assert(lay);
+	lay->setContentsMargins (0,0,0,0);
+	lay->setSpacing (3);
+	
     int ind, lig,col;
-    
-    QFrame *ftmp, *frline;
-
 	//------------------------------------------------
 	// Geographic area
 	//------------------------------------------------
@@ -486,42 +525,38 @@ QFrame *DialogLoadGRIB::createFrameButtonsZone(QWidget *parent)
 	//------------------------------------------------
 	// Choix des données météo
 	//------------------------------------------------
-    chkWind     = new QCheckBox(tr("Vent (10 m)"));
+    chkWind     = new QCheckBox(tr("Wind (10 m)"));
     assert(chkWind);
-    chkPressure = new QCheckBox(tr("Pression au niveau de la mer"));
+    chkPressure = new QCheckBox(tr("Mean sea level pressure"));
     assert(chkPressure);
-    chkRain     = new QCheckBox(tr("Cumul de précipitations"));
+    chkRain     = new QCheckBox(tr("Total precipitation"));
     assert(chkRain);
-    chkCloud     = new QCheckBox(tr("Nébulosité"));
+    chkCloud     = new QCheckBox(tr("Cloud cover"));
     assert(chkCloud);
-    chkTemp     = new QCheckBox(tr("Température (2 m)"));
+    chkTemp     = new QCheckBox(tr("Temperature (2 m)"));
     assert(chkTemp);
-    chkHumid    = new QCheckBox(tr("Humidité relative (2 m)"));
+    chkHumid    = new QCheckBox(tr("Relative humidity (2 m)"));
     assert(chkHumid);
-    chkIsotherm0    = new QCheckBox(tr("Isotherme 0°C"));
+    chkIsotherm0    = new QCheckBox(tr("Isotherm 0°C"));
     assert(chkIsotherm0);
     
-    chkTempPot     = new QCheckBox(tr("Température potentielle (sigma 995)"));
+    chkTempPot     = new QCheckBox(tr("Potential temperature (sigma 995)"));
     assert(chkTempPot);
-    chkTempMin     = new QCheckBox(tr("Température min (2 m)"));
+    chkTempMin     = new QCheckBox(tr("Temperature min (2 m)"));
     assert(chkTempMin);
-    chkTempMax     = new QCheckBox(tr("Température max (2 m)"));
+    chkTempMax     = new QCheckBox(tr("Temperature max (2 m)"));
     assert(chkTempMax);
-    chkSnowCateg     = new QCheckBox(tr("Neige (chute possible)"));
+    chkSnowCateg     = new QCheckBox(tr("Snow (snowfall possible)"));
     assert(chkSnowCateg);
-    chkFrzRainCateg     = new QCheckBox(tr("Pluie verglaçante (chute possible)"));
+    chkFrzRainCateg     = new QCheckBox(tr("Frozen rain (rainfall possible)"));
     assert(chkFrzRainCateg);
-    chkSnowDepth     = new QCheckBox(tr("Neige (épaisseur)"));
+    chkSnowDepth     = new QCheckBox(tr("Snow (depth)"));
     assert(chkSnowDepth);
     chkCAPEsfc     = new QCheckBox(tr("CAPE (surface)"));
     assert(chkCAPEsfc);
 
-    chkAltitudeData  = new QCheckBox(
-    	tr("Atmosphère (850 mb, 700 mb, 500 mb) : altitude, température, humidité spécifique.")
-    	+"\n"
-    	+tr("Attention : ces données augmentent sensiblement la taille du fichier GRIB."));
-    assert(chkAltitudeData);
-    
+	
+    //--------------------------------------------------------------------------------
     chkWind->setChecked    (Util::getSetting("downloadWind", true).toBool());
     chkPressure->setChecked(Util::getSetting("downloadPressure", true).toBool());
     chkRain->setChecked    (Util::getSetting("downloadRain", true).toBool());
@@ -530,22 +565,47 @@ QFrame *DialogLoadGRIB::createFrameButtonsZone(QWidget *parent)
     chkHumid->setChecked   (Util::getSetting("downloadHumid", true).toBool());
     chkIsotherm0->setChecked  (Util::getSetting("downloadIsotherm0", true).toBool());
     
-    chkTempPot->setChecked    (Util::getSetting("downloadTempPot", true).toBool());
-    chkTempMin->setChecked    (Util::getSetting("downloadTempMin", true).toBool());
-    chkTempMax->setChecked    (Util::getSetting("downloadTempMax", true).toBool());
+//    chkTempPot->setChecked    (Util::getSetting("downloadTempPot", false).toBool());
+    chkTempPot->setChecked    (false);
+	
+    chkTempMin->setChecked    (Util::getSetting("downloadTempMin", false).toBool());
+    chkTempMax->setChecked    (Util::getSetting("downloadTempMax", false).toBool());
     chkSnowDepth->setChecked  (Util::getSetting("downloadSnowDepth", true).toBool());
     chkSnowCateg->setChecked  (Util::getSetting("downloadSnowCateg", true).toBool());
-    chkCAPEsfc->setChecked  (Util::getSetting("downloadCAPEsfc", false).toBool());
+    chkCAPEsfc->setChecked  (Util::getSetting("downloadCAPEsfc", true).toBool());
     chkFrzRainCateg->setChecked  (Util::getSetting("downloadFrzRainCateg", true).toBool());
+	
+	//----------------------------------------------------------------
+    chkAltitude850  = new QCheckBox ("850 "+tr("hPa"));
+    assert (chkAltitude850);
+    chkAltitude850->setChecked  (Util::getSetting("downloadAltitudeData850", false).toBool());
+    chkAltitude700  = new QCheckBox ("700 "+tr("hPa"));
+    assert (chkAltitude700);
+    chkAltitude700->setChecked  (Util::getSetting("downloadAltitudeData700", false).toBool());
+    chkAltitude500  = new QCheckBox ("500 "+tr("hPa"));
+    assert (chkAltitude500);
+    chkAltitude500->setChecked  (Util::getSetting("downloadAltitudeData500", false).toBool());
+    chkAltitude300  = new QCheckBox ("300 "+tr("hPa"));
+    assert (chkAltitude300);
+    chkAltitude300->setChecked  (Util::getSetting("downloadAltitudeData300", false).toBool());
+    chkAltitude200  = new QCheckBox ("200 "+tr("hPa"));
+    assert (chkAltitude200);
+    chkAltitude200->setChecked  (Util::getSetting("downloadAltitudeData200", false).toBool());
 
-    chkAltitudeData->setChecked  (Util::getSetting("downloadAltitudeData", false).toBool());
+    chkAltitudeAll = new QCheckBox (tr("All"));
+    assert (chkAltitudeAll);
+	chkAltitudeAll->setChecked  (
+		      chkAltitude200->isChecked() && chkAltitude300->isChecked() 
+		   && chkAltitude500->isChecked() && chkAltitude700->isChecked() 
+		   && chkAltitude850->isChecked()
+		);
 
 	//----------------------------------------------------------------
-    btOK     = new QPushButton(tr("Télécharger le fichier GRIB"), this);
+    btOK     = new QPushButton(tr("Download GRIB file"), this);
     assert(btOK);
-    btCancel = new QPushButton(tr("Annuler"), this);
+    btCancel = new QPushButton(tr("Cancel"), this);
     assert(btCancel);
-    btServerStatus = new QPushButton(tr("Statut du serveur"), this);
+    btServerStatus = new QPushButton(tr("Server status"), this);
     assert(btServerStatus);
 
     progressBar = new QProgressBar();
@@ -560,6 +620,7 @@ QFrame *DialogLoadGRIB::createFrameButtonsZone(QWidget *parent)
 	ftmp = new QFrame(this);
     tgrid = new QGridLayout(ftmp);
     assert(tgrid);
+	tgrid->setContentsMargins (0,0,0,0);
 		tgrid->addWidget( new QLabel(tr("Latitude min :")), 0, 0, Qt::AlignRight);
 		tgrid->addWidget( sbNorth, 0, 1);
 		tgrid->addWidget( new QLabel(tr("Latitude max :")), 0, 2, Qt::AlignRight);
@@ -573,36 +634,39 @@ QFrame *DialogLoadGRIB::createFrameButtonsZone(QWidget *parent)
     lay->addWidget( ftmp);
     
     //-------------------------
-    ftmp = new QFrame(this); ftmp->setFrameShape(QFrame::HLine); lay->addWidget( ftmp);
+    addSeparator (lay, 'H'); 
     //-------------------------
 	
 	ftmp = new QFrame(this);
     tlay = new QHBoxLayout(ftmp);
     assert(tlay);
+	tlay->setContentsMargins (0,0,0,0);
 		
-		tlay->addWidget( new QLabel(tr("Résolution :")));
+		tlay->addWidget( new QLabel(tr("Resolution :")));
 		tlay->addWidget( cbResolution);
 		tlay->addWidget( new QLabel(tr(" °")));
 		//-------------------------
-		frline=new QFrame(this); frline->setFrameShape(QFrame::VLine);tlay->addWidget(frline);
+		addSeparator (tlay, 'V'); 
 		//-------------------------
-		tlay->addWidget( new QLabel(tr("Intervalle :")));
+		tlay->addWidget( new QLabel(tr("Interval :")));
 		tlay->addWidget( cbInterval);
-		tlay->addWidget( new QLabel(tr(" heures")));
+		tlay->addWidget( new QLabel(tr(" hours")));
 		//-------------------------
-		frline=new QFrame(this); frline->setFrameShape(QFrame::VLine);tlay->addWidget(frline);
+		addSeparator (tlay, 'V'); 
 		//-------------------------
-		tlay->addWidget( new QLabel(tr("Durée :")));
+		tlay->addWidget( new QLabel(tr("Period :")));
 		tlay->addWidget( cbDays);
-		tlay->addWidget( new QLabel(tr(" jours")));
+		tlay->addWidget( new QLabel(tr(" days")));
     lay->addWidget( ftmp);
     
     //-------------------------
-    ftmp = new QFrame(this); ftmp->setFrameShape(QFrame::HLine); lay->addWidget( ftmp);
+    addSeparator (lay, 'H'); 
     //-------------------------
 	ftmp = new QFrame(this);
     tgrid = new QGridLayout(ftmp);
     assert(tgrid);
+	tgrid->setContentsMargins (0,0,0,0);
+	tgrid->setSpacing (0);
     	//-----------------------------
     	// Colonne 1
     	//-----------------------------
@@ -613,39 +677,62 @@ QFrame *DialogLoadGRIB::createFrameButtonsZone(QWidget *parent)
 		tgrid->addWidget( chkTemp ,      lig++, col, Qt::AlignLeft);
 		tgrid->addWidget( chkTempMin ,   lig++, col, Qt::AlignLeft);
 		tgrid->addWidget( chkTempMax ,   lig++, col, Qt::AlignLeft);
-		tgrid->addWidget( chkTempPot , lig++, col, Qt::AlignLeft);
-		tgrid->addWidget( chkFrzRainCateg , lig++, col, Qt::AlignLeft);
+		//tgrid->addWidget( chkTempPot , lig++, col, Qt::AlignLeft);
+		tgrid->addWidget( chkIsotherm0 , lig++, col, Qt::AlignLeft);
+		tgrid->addWidget( chkCAPEsfc ,  lig++, col, Qt::AlignLeft);
     	//-----------------------------
     	// Colonne 2
     	//-----------------------------
     	col = 1;
     	lig = 0;
-		tgrid->addWidget( chkIsotherm0 , lig++, col, Qt::AlignLeft);
 		tgrid->addWidget( chkCloud ,   lig++, col, Qt::AlignLeft);
 		tgrid->addWidget( chkHumid ,   lig++, col, Qt::AlignLeft);
 		tgrid->addWidget( chkRain ,    lig++, col, Qt::AlignLeft);
 		tgrid->addWidget( chkSnowCateg ,  lig++, col, Qt::AlignLeft);
 		tgrid->addWidget( chkSnowDepth ,  lig++, col, Qt::AlignLeft);
-		tgrid->addWidget( chkCAPEsfc ,  lig++, col, Qt::AlignLeft);
-		
-		//tgrid->addWidget( chkAltitudeData , lig++, 0,1,-1, Qt::AlignLeft);
+		tgrid->addWidget( chkFrzRainCateg , lig++, col, Qt::AlignLeft);
     lay->addWidget( ftmp);
     
     //-------------------------
-    ftmp = new QFrame(this); ftmp->setFrameShape(QFrame::HLine); lay->addWidget( ftmp);
+    addSeparator (lay, 'H'); 
+    //-------------------------
+	lay->addWidget (new QLabel (
+			tr("Atmosphere: geopotential altitude, wind, temperature, theta-e, relative humidity.") 
+			+"\n"+
+			tr("Warning : these data increase strongly the size of the GRIB file.")
+		));
+	//lay->addWidget( chkAltitudeData );
+	ftmp = new QFrame(this);
+	assert (ftmp);
+		tlay = new QHBoxLayout(ftmp);
+		assert (tlay);
+		tlay->setContentsMargins (0,0,0,0);
+		tlay->addWidget (chkAltitude850);
+		tlay->addWidget (chkAltitude700);
+		tlay->addWidget (chkAltitude500);
+		tlay->addWidget (chkAltitude300);
+		tlay->addWidget (chkAltitude200);
+		addSeparator (tlay, 'V'); 
+		addSeparator (tlay, 'V'); 
+		tlay->addWidget (chkAltitudeAll);
+	lay->addWidget (ftmp);
+	
+    //-------------------------
+    addSeparator (lay, 'H'); 
     //-------------------------
     lay->addWidget( progressBar );
     //-------------------------
     labelMsg = new QLabel();
     lay->addWidget( labelMsg );
-    lay->addWidget(new QLabel(tr("La taille des fichiers est limitée à 20000 ko.")));
-    
+    //lay->addWidget(new QLabel(tr("File size max: 20000 ko.")));
+    lay->addWidget(new QLabel(tr("File size max: ")+ "51200 Ko (50 Mo)"));
     //-------------------------
-    ftmp = new QFrame(this); ftmp->setFrameShape(QFrame::HLine); lay->addWidget( ftmp);
+    addSeparator (lay, 'H'); 
     //-------------------------
 	ftmp = new QFrame(this);
     tlay = new QHBoxLayout(ftmp);
     assert(tlay);
+	tlay->setContentsMargins (0,0,0,0);
 		tlay->addWidget( btOK );
 		tlay->addWidget( btServerStatus );
 		tlay->addWidget( btCancel );
@@ -653,6 +740,21 @@ QFrame *DialogLoadGRIB::createFrameButtonsZone(QWidget *parent)
 
     return frm;
 }
+//----------------------------------------------------------------------------
+void DialogLoadGRIB::addSeparator (QLayout *layout, char orientation)
+{
+	QFrame *ftmp;
+	ftmp = new QFrame ();
+	assert (ftmp);
+	if (orientation == 'H') {
+		ftmp->setFrameShape(QFrame::HLine);
+	}
+	else {
+		ftmp->setFrameShape(QFrame::VLine);
+	}
+	ftmp->setStyleSheet ("color:#AAAAAA");
+	layout->addWidget(ftmp);
+}
 
 
 
@@ -662,5 +764,3 @@ QFrame *DialogLoadGRIB::createFrameButtonsZone(QWidget *parent)
 
 
 
-
-
