@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,19 +16,25 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/
 
-#include "GribReader.h"
 #include <cassert>
 
+#include "GribReader.h"
+#include "Util.h"
+
 //-------------------------------------------------------------------------------
 GribReader::GribReader()
 {
     ok = false;
+	hasAltitude = false;
+	ambiguousHeader = false;
 	dewpointDataStatus = NO_DATA_IN_FILE;
 }
 //-------------------------------------------------------------------------------
 GribReader::GribReader(const std::string fname)
 {
     ok = false;
+	hasAltitude = false;
+	ambiguousHeader = false;
 	dewpointDataStatus = NO_DATA_IN_FILE;
     if (fname != "") {
         openFile(fname);
@@ -43,7 +49,7 @@ GribReader::~GribReader()
     clean_all_vectors();
 }
 //-------------------------------------------------------------------------------
-void GribReader::clean_all_vectors()
+void GribReader::clean_all_vectors ()
 {
 	std::map < std::string, std::vector<GribRecord *>* >::iterator it;
 	for (it=mapGribRecords.begin(); it!=mapGribRecords.end(); it++) {
@@ -54,7 +60,7 @@ void GribReader::clean_all_vectors()
 	mapGribRecords.clear();
 }
 //-------------------------------------------------------------------------------
-void GribReader::clean_vector(std::vector<GribRecord *> &ls)
+void GribReader::clean_vector (std::vector<GribRecord *> &ls)
 {
     std::vector<GribRecord *>::iterator it;
     for (it=ls.begin(); it!=ls.end(); it++) {
@@ -64,20 +70,39 @@ void GribReader::clean_vector(std::vector<GribRecord *> &ls)
     ls.clear();
 }
 
-//---------------------------------------------------------------------------------    
-void GribReader::storeRecordInMap(GribRecord *rec)
+//---------------------------------------------------------------------------------
+void GribReader::storeRecordInMap (GribRecord *rec)
 {
+	if (rec==NULL)
+		return;
 	std::map <std::string, std::vector<GribRecord *>* >::iterator it;
-	it = mapGribRecords.find(rec->getKey());	
+	it = mapGribRecords.find(rec->getKey());
 	if (it == mapGribRecords.end())
 	{
 		mapGribRecords[rec->getKey()] = new std::vector<GribRecord *>;
 		assert(mapGribRecords[rec->getKey()]);
-	}	
+	}
 	mapGribRecords[rec->getKey()]->push_back(rec);
+	
+	// Update list of records types
+	setAllDataCode.insert (rec->getDataCode());
+	if (rec->getLevelType()==LV_ISOBARIC
+			&& (   rec->getLevelValue()==850
+				|| rec->getLevelValue()==700
+				|| rec->getLevelValue()==500
+				|| rec->getLevelValue()==300 
+				|| rec->getLevelValue()==200 
+				)
+	) {
+		hasAltitude = true;
+	}
+	
+	if (rec->isOrientationAmbiguous()) {
+		ambiguousHeader = true;
+	}
 }
 
-//---------------------------------------------------------------------------------    
+//---------------------------------------------------------------------------------
 void GribReader::readAllGribRecords()
 {
     //--------------------------------------------------------
@@ -89,17 +114,19 @@ void GribReader::readAllGribRecords()
     time_t firstdate = -1;
     do {
         id ++;
+		
         rec = new GribRecord(file, id);
         assert(rec);
-        if (rec->isOk())
+        
+		if (rec->isOk())
         {
         	if (rec->isDataKnown())
         	{
 				ok = true;   // au moins 1 record ok
-				
+
 				if (firstdate== -1)
 					firstdate = rec->getRecordCurrentDate();
-				
+
 
 				if (//-----------------------------------------
 						(rec->getDataType()==GRB_PRESSURE
@@ -115,24 +142,39 @@ void GribReader::readAllGribRecords()
 							&& (   rec->getLevelValue()==850
 								|| rec->getLevelValue()==700
 								|| rec->getLevelValue()==500
-								|| rec->getLevelValue()==300 ) )
+								|| rec->getLevelValue()==300 
+								|| rec->getLevelValue()==200 ) )
+					//-----------------------------------------
+					// Wind
 					//-----------------------------------------
 					|| ( (rec->getDataType()==GRB_WIND_VX || rec->getDataType()==GRB_WIND_VY)
 							&& rec->getLevelType()==LV_ABOV_GND
-							&& rec->getLevelValue()==10)
+							&& (   rec->getLevelValue()==1
+								|| rec->getLevelValue()==2
+								|| rec->getLevelValue()==3
+								|| rec->getLevelValue()==10 ) )
+					|| ( (rec->getDataType()==GRB_WIND_VX || rec->getDataType()==GRB_WIND_VY)
+							&& rec->getLevelType()==LV_MSL
+							&& rec->getLevelValue()==0 )
+					|| ( (rec->getDataType()==GRB_WIND_VX || rec->getDataType()==GRB_WIND_VY)
+							&& rec->getLevelType()==LV_GND_SURF
+							&& rec->getLevelValue()==0 )
 					|| ( (rec->getDataType()==GRB_WIND_VX || rec->getDataType()==GRB_WIND_VY)
 							&& rec->getLevelType()==LV_ISOBARIC
 							&& (   rec->getLevelValue()==850
 								|| rec->getLevelValue()==700
 								|| rec->getLevelValue()==500
-								|| rec->getLevelValue()==300 ) )
+								|| rec->getLevelValue()==300 
+								|| rec->getLevelValue()==200 ) )
+					//-----------------------------------------
 					//-----------------------------------------
 					|| (rec->getDataType()==GRB_HUMID_SPEC
 							&& rec->getLevelType()==LV_ISOBARIC
 							&& (   rec->getLevelValue()==850
 								|| rec->getLevelValue()==700
 								|| rec->getLevelValue()==500
-								|| rec->getLevelValue()==300 ) )
+								|| rec->getLevelValue()==300 
+								|| rec->getLevelValue()==200 ) )
 					//-----------------------------------------
 					|| (rec->getDataType()==GRB_GEOPOT_HGT
 							&& rec->getLevelType()==LV_ISOTHERM0 && rec->getLevelValue()==0)
@@ -141,7 +183,8 @@ void GribReader::readAllGribRecords()
 							&& (   rec->getLevelValue()==850
 								|| rec->getLevelValue()==700
 								|| rec->getLevelValue()==500
-								|| rec->getLevelValue()==300 ) )
+								|| rec->getLevelValue()==300 
+								|| rec->getLevelValue()==200 ) )
 					//-----------------------------------------
 					|| (rec->getDataType()==GRB_PRECIP_TOT
 							&& rec->getLevelType()==LV_GND_SURF && rec->getLevelValue()==0)
@@ -163,19 +206,27 @@ void GribReader::readAllGribRecords()
 					//-----------------------------------------
 					|| (rec->getDataType()==GRB_HUMID_REL
 							&& rec->getLevelType()==LV_ABOV_GND && rec->getLevelValue()==2)
+					|| (rec->getDataType()==GRB_HUMID_REL
+							&& rec->getLevelType()==LV_ISOBARIC
+							&& (   rec->getLevelValue()==850
+								|| rec->getLevelValue()==700
+								|| rec->getLevelValue()==500
+								|| rec->getLevelValue()==300 
+								|| rec->getLevelValue()==200 ) )
 					//-----------------------------------------
-					|| (rec->getDataType()==GRB_TPOT
+					|| (rec->getDataType()==GRB_TEMP_POT
 							&& rec->getLevelType()==LV_SIGMA && rec->getLevelValue()==9950)
 					//-----------------------------------------
 					|| (rec->getDataType()==GRB_CAPE
 							&& rec->getLevelType()==LV_GND_SURF && rec->getLevelValue()==0)
 				)
 				{
-					storeRecordInMap(rec);
+					storeRecordInMap (rec);
 				}
 				else {
 					fprintf(stderr,
-						"GribReader: unknown record type: key=%s  idCenter==%d && idModel==%d && idGrid==%d\n",
+						"GribReader: id=%d unknown data: key=%s  idCenter==%d && idModel==%d && idGrid==%d\n",
+						rec->getId(),
 						rec->getKey().c_str(),
 						rec->getIdCenter(), rec->getIdModel(), rec->getIdGrid()
 						);
@@ -187,36 +238,37 @@ void GribReader::readAllGribRecords()
             rec = NULL;
         }
     } while (rec != NULL &&  !rec->isEof());
+
 }
 
 
-//---------------------------------------------------------------------------------    
-void  GribReader::copyFirstCumulativeRecord (int dataType,int levelType,int levelValue)
+//---------------------------------------------------------------------------------
+void  GribReader::copyFirstCumulativeRecord (DataCode dtc)
 {
 	time_t dateref = getRefDate();
-	GribRecord *rec = getGribRecord(dataType, levelType, levelValue, dateref);
+	GribRecord *rec = getRecord (dtc, dateref);
 	if (rec == NULL)
 	{
-		rec = getFirstGribRecord(dataType, levelType, levelValue);
+		rec = getFirstGribRecord (dtc);
 		if (rec != NULL)
 		{
-			GribRecord *r2 = new GribRecord(*rec);
+			GribRecord *r2 = new GribRecord (*rec);
 			if (r2 != NULL) {
 				r2->setRecordCurrentDate (dateref);    // 1er enregistrement factice
-				storeRecordInMap(r2);
+				storeRecordInMap (r2);
 			}
 		}
 	}
 }
-//---------------------------------------------------------------------------------    
-void  GribReader::removeFirstCumulativeRecord (int dataType,int levelType,int levelValue)
+//---------------------------------------------------------------------------------
+void  GribReader::removeFirstCumulativeRecord (DataCode dtc)
 {
 	time_t dateref = getRefDate();
-	GribRecord *rec = getFirstGribRecord(dataType, levelType, levelValue);
-	
+	GribRecord *rec = getFirstGribRecord (dtc);
+
 	if (rec!=NULL  &&  rec->getRecordCurrentDate() == dateref)
 	{
-		std::vector<GribRecord *> *liste = getListOfGribRecords(dataType, levelType, levelValue);		
+		std::vector<GribRecord *> *liste = getListOfGribRecords (dtc);
 		if (liste != NULL) {
 			std::vector<GribRecord *>::iterator it;
 			for (it=liste->begin(); it!=liste->end() && (*it)!=rec; it++)
@@ -231,74 +283,99 @@ void  GribReader::removeFirstCumulativeRecord (int dataType,int levelType,int le
 //---------------------------------------------------------------------------------
 void  GribReader::copyFirstCumulativeRecord()
 {
-	copyFirstCumulativeRecord(GRB_TMIN, LV_ABOV_GND, 2);
-	copyFirstCumulativeRecord(GRB_TMAX, LV_ABOV_GND, 2);
-    copyFirstCumulativeRecord(GRB_CLOUD_TOT,   LV_ATMOS_ALL, 0);
-	copyFirstCumulativeRecord(GRB_PRECIP_TOT,  LV_GND_SURF, 0);
-	copyFirstCumulativeRecord(GRB_PRECIP_RATE, LV_GND_SURF, 0);
-	copyFirstCumulativeRecord(GRB_SNOW_CATEG,  LV_GND_SURF, 0);
-	copyFirstCumulativeRecord(GRB_FRZRAIN_CATEG, LV_GND_SURF, 0);
+	copyFirstCumulativeRecord (DataCode(GRB_TMIN, LV_ABOV_GND, 2));
+	copyFirstCumulativeRecord (DataCode(GRB_TMAX, LV_ABOV_GND, 2));
+    copyFirstCumulativeRecord (DataCode(GRB_CLOUD_TOT,   LV_ATMOS_ALL, 0));
+	copyFirstCumulativeRecord (DataCode(GRB_PRECIP_TOT,  LV_GND_SURF, 0));
+	copyFirstCumulativeRecord (DataCode(GRB_PRECIP_RATE, LV_GND_SURF, 0));
+	copyFirstCumulativeRecord (DataCode(GRB_SNOW_CATEG,  LV_GND_SURF, 0));
+	copyFirstCumulativeRecord (DataCode(GRB_FRZRAIN_CATEG, LV_GND_SURF, 0));
 }
-//---------------------------------------------------------------------------------    
+//---------------------------------------------------------------------------------
 void  GribReader::removeFirstCumulativeRecord()
 {
-	removeFirstCumulativeRecord(GRB_TMIN, LV_ABOV_GND, 2);
-	removeFirstCumulativeRecord(GRB_TMAX, LV_ABOV_GND, 2);
-    removeFirstCumulativeRecord(GRB_CLOUD_TOT,   LV_ATMOS_ALL, 0);
-	removeFirstCumulativeRecord(GRB_PRECIP_TOT,  LV_GND_SURF, 0);
-	removeFirstCumulativeRecord(GRB_PRECIP_RATE, LV_GND_SURF, 0);
-	removeFirstCumulativeRecord(GRB_SNOW_CATEG,  LV_GND_SURF, 0);
-	removeFirstCumulativeRecord(GRB_FRZRAIN_CATEG, LV_GND_SURF, 0);
+	removeFirstCumulativeRecord (DataCode(GRB_TMIN, LV_ABOV_GND, 2));
+	removeFirstCumulativeRecord (DataCode(GRB_TMAX, LV_ABOV_GND, 2));
+    removeFirstCumulativeRecord (DataCode(GRB_CLOUD_TOT,   LV_ATMOS_ALL, 0));
+	removeFirstCumulativeRecord (DataCode(GRB_PRECIP_TOT,  LV_GND_SURF, 0));
+	removeFirstCumulativeRecord (DataCode(GRB_PRECIP_RATE, LV_GND_SURF, 0));
+	removeFirstCumulativeRecord (DataCode(GRB_SNOW_CATEG,  LV_GND_SURF, 0));
+	removeFirstCumulativeRecord (DataCode(GRB_FRZRAIN_CATEG, LV_GND_SURF, 0));
 }
 
-//---------------------------------------------------------------------------------    
+//---------------------------------------------------------------------------------
 void GribReader::readGribFileContent()
 {
     fileSize = zu_filesize(file);
+	
     readAllGribRecords();
-    
     createListDates();
-    hoursBetweenRecords = computeHoursBeetweenGribRecords();
 
 	//-----------------------------------------------------
 	// Are dewpoint data in file ?
 	// If no, compute it with Magnus-Tetens formula, if possible.
 	//-----------------------------------------------------
 	dewpointDataStatus = DATA_IN_FILE;
-	if (getNumberOfGribRecords(GRB_DEWPOINT, LV_ABOV_GND, 2) == 0)
+	if (getNumberOfGribRecords (DataCode(GRB_DEWPOINT, LV_ABOV_GND, 2)) == 0)
 	{
 		dewpointDataStatus = NO_DATA_IN_FILE;
-		if (  getNumberOfGribRecords(GRB_HUMID_REL, LV_ABOV_GND, 2) > 0
-		   && getNumberOfGribRecords(GRB_TEMP, LV_ABOV_GND, 2) > 0)
+		if (  getNumberOfGribRecords (DataCode(GRB_HUMID_REL, LV_ABOV_GND, 2)) > 0
+		   && getNumberOfGribRecords (DataCode(GRB_TEMP, LV_ABOV_GND, 2)) > 0)
 		{
 			dewpointDataStatus = COMPUTED_DATA;
 			std::set<time_t>::iterator iter;
 			for (iter=setAllDates.begin(); iter!=setAllDates.end(); iter++)
 			{
 				time_t date = *iter;
-				GribRecord *recModel = getGribRecord(GRB_TEMP,LV_ABOV_GND,2,date);
+				GribRecord *recModel = getRecord (DataCode(GRB_TEMP,LV_ABOV_GND,2),date);
 				if (recModel != NULL)
 				{
 					// Crée un GribRecord avec les dewpoints calculés
 					GribRecord *recDewpoint = new GribRecord(*recModel);
-					if (recDewpoint != NULL)
+					if (recDewpoint != NULL) 
 					{
-						recDewpoint->setDataType(GRB_DEWPOINT);
-						for (zuint i=0; i<(zuint)recModel->getNi(); i++)
-							for (zuint j=0; j<(zuint)recModel->getNj(); j++)
+						recDewpoint->setDataType (GRB_DEWPOINT);
+						for (int i=0; i<recModel->getNi(); i++)
+						{
+							for (int j=0; j<recModel->getNj(); j++)
 							{
 								double x = recModel->getX(i);
 								double y = recModel->getY(j);
 								double dp = computeDewPoint(x, y, date);
 								recDewpoint->setValue(i, j, dp);
 							}
+						}
 						storeRecordInMap(recDewpoint);
 					}
 				}
 			}
 		}
 	}
-	//-----------------------------------------------------
+}
+
+//-------------------------------------------------------
+double GribReader::computeDewPoint(double lon, double lat, time_t now)
+{
+	double dewpoint = GRIB_NOTDEF;
+
+	GribRecord *recTempDew =  getRecord (DataCode(GRB_DEWPOINT,LV_ABOV_GND,2), now);
+	if (recTempDew != NULL)
+	{
+		// GRIB file contains dew point data
+		dewpoint = recTempDew->getInterpolatedValue(lon, lat);
+	}
+	else
+	{
+		// Compute dew point with Magnus-Tetens formula
+		GribRecord *recTemp =  getRecord (DataCode(GRB_TEMP,LV_ABOV_GND,2), now);
+		GribRecord *recHumid = getRecord (DataCode(GRB_HUMID_REL,LV_ABOV_GND,2), now);
+		if (recTemp && recHumid) {
+			double temp = recTemp->getInterpolatedValue   (lon, lat);
+			double humid = recHumid->getInterpolatedValue (lon, lat);
+			dewpoint = DataRecordAbstract::dewpointMagnusTetens (temp, humid);
+		}
+	}
+	return dewpoint;
 }
 
 //---------------------------------------------------
@@ -333,9 +410,9 @@ std::vector<GribRecord *> * GribReader::getFirstNonEmptyList()
 }
 
 //---------------------------------------------------
-int GribReader::getNumberOfGribRecords(int dataType,int levelType,int levelValue)
+int GribReader::getNumberOfGribRecords (DataCode dtc)
 {
-	std::vector<GribRecord *> *liste = getListOfGribRecords(dataType,levelType,levelValue);
+	std::vector<GribRecord *> *liste = getListOfGribRecords (dtc);
 	if (liste != NULL)
 		return liste->size();
 	else
@@ -343,28 +420,45 @@ int GribReader::getNumberOfGribRecords(int dataType,int levelType,int levelValue
 }
 
 //---------------------------------------------------------------------
-std::vector<GribRecord *> * GribReader::getListOfGribRecords(int dataType,int levelType,int levelValue)
+std::vector<GribRecord *> * GribReader::getListOfGribRecords (DataCode dtc)
 {
-	std::string key = GribRecord::makeKey(dataType,levelType,levelValue);
+	std::string key = GribRecord::makeKey (dtc.dataType,dtc.levelType,dtc.levelValue);
 	if (mapGribRecords.find(key) != mapGribRecords.end())
 		return mapGribRecords[key];
 	else
 		return NULL;
 }
 //---------------------------------------------------------------------------
-double  GribReader::getTimeInterpolatedValue(int dataType,int levelType,int levelValue, double px, double py, time_t date)
+double  GribReader::getDateInterpolatedValue (
+						DataCode dtc, double px, double py, time_t date)
+{
+	if (dtc.dataType == GRB_DEWPOINT) {
+		if (dtc.levelType==LV_ABOV_GND && dtc.levelValue==2)
+			return computeDewPoint(px, py, date);
+	}
+	else {
+		GribRecord *rec;
+		if ( (rec = getRecord (dtc, date)) != NULL)
+			return rec->getInterpolatedValue (px, py);
+	}
+	return GRIB_NOTDEF;
+}
+
+//---------------------------------------------------------------------------
+double  GribReader::get2DatesInterpolatedValue (
+				DataCode dtc, double px, double py, time_t date)
 {
 	GribRecord *before, *after;
-	findGribsAroundDate (dataType,levelType,levelValue, date, &before, &after);
-	return get2GribsInterpolatedValueByDate(px, py, date, before, after);
+	findGribsAroundDate (dtc, date, &before, &after);
+	return get2GribsInterpolatedValueByDate (px, py, date, before, after);
 }
 
 //------------------------------------------------------------------
-void GribReader::findGribsAroundDate (int dataType,int levelType,int levelValue, time_t date,
+void GribReader::findGribsAroundDate (DataCode dtc, time_t date,
 							GribRecord **before, GribRecord **after)
 {
 	// Cherche les GribRecord qui encadrent la date
-	std::vector<GribRecord *> *ls = getListOfGribRecords(dataType,levelType,levelValue);
+	std::vector<GribRecord *> *ls = getListOfGribRecords (dtc);
 	*before = NULL;
 	*after  = NULL;
 	zuint nb = ls->size();
@@ -409,13 +503,13 @@ double 	GribReader::get2GribsInterpolatedValueByDate (
 				}
 			}
 		}
-	}	
+	}
 	return val;
 }
 
 //---------------------------------------------------
 // Rectangle de la zone couverte par les données
-bool GribReader::getZoneExtension(double *x0,double *y0, double *x1,double *y1)
+bool GribReader::getZoneExtension (double *x0,double *y0, double *x1,double *y1)
 {
     std::vector<GribRecord *> *ls = getFirstNonEmptyList();
     if (ls != NULL) {
@@ -456,37 +550,21 @@ GribRecord * GribReader::getFirstGribRecord()
 }
 //---------------------------------------------------
 // Premier GribRecord (par date) pour un type donné
-GribRecord * GribReader::getFirstGribRecord(int dataType,int levelType,int levelValue)
+GribRecord * GribReader::getFirstGribRecord (DataCode dtc)
 {
 	std::set<time_t>::iterator it;
 	GribRecord *rec = NULL;
 	for (it=setAllDates.begin(); rec==NULL && it!=setAllDates.end(); it++)
 	{
 		time_t date = *it;
-		rec = getGribRecord(dataType,levelType,levelValue, date);
+		rec = getRecord (dtc, date);
 	}
 	return rec;
 }
 //---------------------------------------------------
-// Délai en heures entre 2 records
-// On suppose qu'il est fixe pour tout le fichier !!!
-double GribReader::computeHoursBeetweenGribRecords()
+GribRecord * GribReader::getRecord (DataCode dtc, time_t date)
 {
-	double res = 1;
-    std::vector<GribRecord *> *ls = getFirstNonEmptyList();
-    if (ls != NULL) {
-        time_t t0 = (*ls)[0]->getRecordCurrentDate();
-        time_t t1 = (*ls)[1]->getRecordCurrentDate();
-        res = abs(t1-t0) / 3600.0;
-        if (res < 1)
-        	res = 1;  
-    }
-    return res;
-}
-//---------------------------------------------------
-GribRecord * GribReader::getGribRecord(int dataType,int levelType,int levelValue, time_t date)
-{
-    std::vector<GribRecord *> *ls = getListOfGribRecords(dataType,levelType,levelValue);
+    std::vector<GribRecord *> *ls = getListOfGribRecords (dtc);
     if (ls != NULL) {
         // Cherche le premier enregistrement à la bonne date
         GribRecord *res = NULL;
@@ -517,44 +595,6 @@ void GribReader::createListDates()
 	}
 }
 
-//-------------------------------------------------------
-double GribReader::computeDewPoint(double lon, double lat, time_t now)
-{
-	double diewpoint = GRIB_NOTDEF;
-
-	GribRecord *recTempDiew =  getGribRecord(GRB_DEWPOINT,LV_ABOV_GND,2,now);
-	if (recTempDiew != NULL)
-	{
-		// GRIB file contains diew point data
-		diewpoint = recTempDiew->getInterpolatedValue(lon, lat);
-	}
-	else
-	{	
-		// Compute diew point with Magnus-Tetens formula
-		GribRecord *recTemp =  getGribRecord(GRB_TEMP,LV_ABOV_GND,2,now);
-		GribRecord *recHumid = getGribRecord(GRB_HUMID_REL,LV_ABOV_GND,2,now);
-		if (recTemp && recHumid)
-		{
-			double temp = recTemp->getInterpolatedValue(lon, lat);
-			double humid = recHumid->getInterpolatedValue(lon, lat);
-			if (temp != GRIB_NOTDEF && humid != GRIB_NOTDEF)
-			{
-				double a = 17.27;
-				double b = 237.7;
-				double t  = temp-273.15;
-				double rh = humid;
-				//if ( t>0 && t<60 && rh>0.01)
-				{
-					double alpha = a*t/(b+t)+log(rh/100.0);
-					diewpoint = b*alpha/(a-alpha);
-					diewpoint += 273.15;
-				}
-			}
-		}
-	}
-	return diewpoint;
-}
-
 
 //-------------------------------------------------------------------------------
 // Lecture complète d'un fichier GRIB
@@ -574,28 +614,5 @@ void GribReader::openFile(const std::string fname)
         return;
     }
     readGribFileContent();
-    
-    // Essaie d'autres compressions (extension non reconnue ?)
- 	if (! ok) {
-    	if (file != NULL)
-			zu_close(file);
-    	file = zu_open(fname.c_str(), "rb", ZU_COMPRESS_BZIP);
-    	if (file != NULL)
-    		readGribFileContent();
-    }
- 	if (! ok) {
-    	if (file != NULL)
-			zu_close(file);
-    	file = zu_open(fname.c_str(), "rb", ZU_COMPRESS_GZIP);
-    	if (file != NULL)
-    		readGribFileContent();
-    }
- 	if (! ok) {
-    	if (file != NULL)
-			zu_close(file);
-    	file = zu_open(fname.c_str(), "rb", ZU_COMPRESS_NONE);
-    	if (file != NULL)
-    		readGribFileContent();
-    }
 }
 
