@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,23 +16,21 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/
 
-#include <stdlib.h>
-
-#include <QDateTime>
-
 #include "GribRecord.h"
-		
+
 //-------------------------------------------------------------------------------
 // Adjust data type from different mete center
 //-------------------------------------------------------------------------------
 void  GribRecord::translateDataType()
 {
 	this->knownData = true;
-	
+
 	//------------------------
 	// NOAA GFS
 	//------------------------
-	if (idCenter==7 && (idModel==96 || idModel==81) && idGrid==4)
+	if (   idCenter==7
+		&& (idModel==96 || idModel==81)		// NOAA
+		&& (idGrid==4 || idGrid==255))		// Saildocs, Ocens
 	{
 		if (dataType == GRB_PRECIP_TOT) {	// mm/period -> mm/h
 			if (periodP2 > periodP1)
@@ -42,8 +40,18 @@ void  GribRecord::translateDataType()
 			if (periodP2 > periodP1)
 				multiplyAllData( 3600.0 );
 		}
-
-	
+	}
+	//------------------------
+	// NOAA Waves
+	//------------------------
+	else if (   
+			    (idCenter==7 && idModel==122 && idGrid==239)  // akw.all.grb
+			 || (idCenter==7 && idModel==124 && idGrid==253)  // enp.all.grb
+			 || (idCenter==7 && idModel==123 && idGrid==244)  // nah.all.grb
+			 || (idCenter==7 && idModel==125 && idGrid==253)  // nph.all.grb
+			 || (idCenter==7 && idModel==88 && idGrid==233)	  // nwww3.all.grb
+			 || (idCenter==7 && idModel==121 && idGrid==238)  // wna.all.grb
+	) {
 	}
 	//------------------------
 	// WRF NMM grib.meteorologic.net
@@ -58,12 +66,10 @@ void  GribRecord::translateDataType()
 			if (periodP2 > periodP1)
 				multiplyAllData( 3600.0 );
 		}
-
-	
 	}
-	//------------------------
-	// Meteorem
-	//------------------------
+	//----------------------------------------------
+	// Meteorem (Scannav)
+	//----------------------------------------------
 	else if (idCenter==59 && idModel==78 && idGrid==255)
 	{
 		if ( (getDataType()==GRB_WIND_VX || getDataType()==GRB_WIND_VY)
@@ -71,36 +77,113 @@ void  GribRecord::translateDataType()
 			&& getLevelValue()==0)
 		{
 			levelType  = LV_ABOV_GND;
-			levelValue = 10;
-		}	
+			levelValue = 0;
+		}
 		if ( getDataType()==GRB_PRECIP_TOT
 			&& getLevelType()==LV_MSL
 			&& getLevelValue()==0)
 		{
 			levelType  = LV_GND_SURF;
 			levelValue = 0;
-		}	
+		}
+	}
+	//------------------------------------------
+	// Others recognized grib suppliers
+	//------------------------------------------
+	else if (
+		(idCenter==255 && idModel==1 && idGrid==255) //http://www.navcenter.com/grb/
+	 || (idCenter==7   && idModel==96 && idGrid==3)    // Maxsea
+	 || (idCenter==255 && idModel==255 && idGrid==255) // Maxsea: tide current
+	) {
 	}
 	//------------------------
 	// Unknown center
 	//------------------------
 	else
 	{
-		printf("Unknown GribRecord: ");
-		this->print();
+		this->print("translateDataType() : Unknown GribRecord");
 		this->knownData = false;
-		
 	}
-		//this->print();
+	
+	if (this->knownData) {
+		if ( (getDataType()==GRB_WIND_VX || getDataType()==GRB_WIND_VY)
+			&& getLevelType()== LV_ABOV_GND
+			&& getLevelValue()== 0)
+		{
+			levelType  = LV_GND_SURF;
+			levelValue = 0;
+		}
+		else if ( (getDataType()==GRB_WIND_VX || getDataType()==GRB_WIND_VY)
+			&& getLevelType()== LV_GND_SURF)
+		{
+			levelValue = 0;
+		}
+	}
+}
+
+//----------------------------------------------------------------------------
+// Try to detect ambiguous header from bad grib file providers :(
+//----------------------------------------------------------------------------
+bool GribRecord::verticalDataAreMirrored() {
+	//---------------------------------------------
+	// Malformed Maxsea grib file ?
+	//---------------------------------------------
+	if (    (idCenter==7 && idModel==96 && idGrid==4)   // same ident than NOAA
+	     || (idCenter==7 && idModel==81 && idGrid==4)	// 2nd ident values
+	     || (idCenter==7 && idModel==96 && idGrid==3)	// "oceanic" model
+	     || (idCenter==7 && idModel==88 && idGrid==233)	// "oceanic" model
+	     || (idCenter==255 && idModel==255 && idGrid==255)	// "oceanic" model: tide current
+	) {
+		if ( ! hasDiDj 
+			&& savXmin < savXmax   
+			&& savYmin > savYmax
+			&& Di>0  && Dj>0
+			&& isScanIpositive
+			&& !isScanJpositive
+		   )
+		{
+			verticalOrientationIsAmbiguous = true;
+			return true;
+		}
+	}
+	//---------------------------------------------
+	// Malformed Meteorem (Scannav) grib file ?
+	//---------------------------------------------
+	else if (    (idCenter==59 && idModel==78 && idGrid==255) 
+	) {
+		if ( ! hasDiDj 
+			&& savXmin < savXmax
+			&& savYmin < savYmax
+			&& Di>0  && Dj>0
+			&& isScanIpositive
+			&& !isScanJpositive
+		   )
+		{
+			verticalOrientationIsAmbiguous = true;
+			return true;
+		}
+	}
+	else {
+		print ("verticalDataAreMirrored()");
+	}
+	
+	return false;
 }
 
 //-------------------------------------------------------------------------------
-void GribRecord::print()
+void GribRecord::print (const char *title)
 {
-	printf("%d: idCenter=%d idModel=%d idGrid=%d dataType=%d hr=%f\n",
-			id, idCenter, idModel, idGrid, dataType,
-			(curDate-refDate)/3600.0
-			);
+fprintf(stderr,"====== GribRecord %d : %s\n", id, title);
+fprintf(stderr,"idCenter=%d idModel=%d idGrid=%d\n", idCenter,idModel,idGrid);
+fprintf(stderr,"dataType=%d levelType=%d levelValue=%d\n", dataType, levelType,levelValue);
+fprintf(stderr,"hour=%g\n", (curDate-refDate)/3600.0);
+fprintf(stderr,"sav   xmin=%f xmax=%f    ymin=%f ymax=%f\n", savXmin,savXmax,savYmin,savYmax);
+fprintf(stderr,"final xmin=%f xmax=%f    ymin=%f ymax=%f\n", xmin,xmax,ymin,ymax);
+fprintf(stderr,"Ni=%d Nj=%d\n", Ni,Nj);
+fprintf(stderr,"hasDiDj=%d Di,Dj=(%f %f)\n", hasDiDj, savDi,savDj);
+fprintf(stderr,"final     Di,Dj=(%f %f)\n", Di,Dj);
+fprintf(stderr,"hasBMS=%d isScanIpositive=%d isScanJpositive=%d isAdjacentI=%d\n",
+                       hasBMS, isScanIpositive,isScanJpositive,isAdjacentI );
 }
 
 //-------------------------------------------------------------------------------
@@ -112,9 +195,12 @@ GribRecord::GribRecord(ZUFILE* file, int id_)
     seekStart = zu_tell(file);
     data    = NULL;
     BMSbits = NULL;
+	boolBMStab = NULL;
     eof     = false;
 	knownData = true;
-    
+	editionNumber = 0;
+	verticalOrientationIsAmbiguous = false;
+
     ok = readGribSection0_IS(file);
     if (ok) {
         ok = readGribSection1_PDS(file);
@@ -138,13 +224,24 @@ GribRecord::GribRecord(ZUFILE* file, int id_)
     if (ok) {
         zu_seek(file, seekStart+totalSize, SEEK_SET);
     }
-
+	
+	if (ok && hasBMS) {
+		// replace the BMS bits table with a faster bool table
+        boolBMStab = new bool [Ni*Nj];
+		assert (boolBMStab);
+		for (int i=0; i<Ni; i++) {
+			for (int j=0; j<Nj; j++) {
+				boolBMStab [j*Ni+i] = hasValueInBitBMS (i,j);
+			}
+		}
+	}
+	
+	checkOrientation ();
     if (ok) {
-		translateDataType();
-		setDataType(dataType);
+		translateDataType ();
+		setDataType (dataType);
 	}
 }
-
 //-------------------------------------------------------------------------------
 // Constructeur de recopie
 //-------------------------------------------------------------------------------
@@ -155,17 +252,118 @@ GribRecord::GribRecord(const GribRecord &rec)
     if (rec.data != NULL) {
         int size = rec.Ni*rec.Nj;
         this->data = new double[size];
+		assert (this->data);
         for (int i=0; i<size; i++)
             this->data[i] = rec.data[i];
     }
     if (rec.BMSbits != NULL) {
         int size = rec.sectionSize3-6;
         this->BMSbits = new zuchar[size];
+		assert (this->BMSbits);
         for (int i=0; i<size; i++)
             this->BMSbits[i] = rec.BMSbits[i];
     }
+    if (rec.boolBMStab != NULL) {
+        int size = rec.Ni*rec.Nj;
+        this->boolBMStab = new bool[size];
+		assert (this->boolBMStab);
+        for (int i=0; i<size; i++)
+            this->boolBMStab[i] = rec.boolBMStab[i];
+    }
+	checkOrientation ();
 }
-
+//--------------------------------------------------------------------------
+GribRecord::~GribRecord()
+{
+    if (data) {
+        delete [] data;
+        data = NULL;
+    }
+    if (BMSbits) {
+        delete [] BMSbits;
+        BMSbits = NULL;
+    }
+	if (boolBMStab) {
+        delete [] boolBMStab;
+        boolBMStab = NULL;
+    }
+	
+}
+//------------------------------------------------------------------------------
+void  GribRecord::checkOrientation ()
+{
+	if (!ok || !data || ymin==ymax  ||  xmin==xmax
+		|| Ni<=1 || Nj<=1
+	) {
+		ok = false;
+		return;
+	}
+	double v;
+	if (xmin > xmax) 
+	{
+		reverseData ('H');
+		v=xmin;  xmin=xmax;  xmax=v;
+		Di = fabs(Di);
+	}
+	if (ymin > ymax) 
+	{
+		//printf("GribRecord::checkOrientation (): ymin>ymax => must reverse data\n");		
+		reverseData ('V');
+		v=ymin;  ymin=ymax;  ymax=v;
+		Dj = fabs(Dj);
+	}
+	
+    while (xmin<=-180) {
+        xmin += 360.0;
+        xmax += 360.0;
+    }
+	
+	if (verticalDataAreMirrored()) 
+	{
+		//printf("GribRecord::checkOrientation (): AMBIGUOUS FILE => must reverse data\n");		
+		reverseData ('V');
+		Dj = fabs(Dj);
+	}
+}
+//------------------------------------------------------------------------------
+void GribRecord::reverseData (char orientation) // orientation = 'H' or 'V'
+{
+	int i, j, i1, j1, i2, j2;
+	double v;
+	bool b;
+	if (orientation == 'H') 
+	{
+		for (j=0; j<Nj; j++) {
+			for (i1=0,i2=Ni-1;  i1<i2;  i1++,i2--) // Reverse line j
+			{
+				v = data [j*Ni+i1];
+				data [j*Ni+i1] = data [j*Ni+i2];
+				data [j*Ni+i2] = v;
+				if (hasBMS) {
+					b = boolBMStab [j*Ni+i1];
+					boolBMStab [j*Ni+i1] = boolBMStab [j*Ni+i2];
+					boolBMStab [j*Ni+i2] = b;
+				}
+			}
+		} 
+	}
+	else if (orientation == 'V') 
+	{
+		for (i=0; i<Ni; i++) {
+			for (j1=0,j2=Nj-1;  j1<j2;  j1++,j2--) // Reverse row i
+			{
+				v = data [j1*Ni+i];
+				data [j1*Ni+i] = data [j2*Ni+i];
+				data [j2*Ni+i] = v;
+				if (hasBMS) {
+					b = boolBMStab [j1*Ni+i];
+					boolBMStab [j1*Ni+i] = boolBMStab [j2*Ni+i];
+					boolBMStab [j2*Ni+i] = b;
+				}
+			}
+		}
+	}
+}	
 //------------------------------------------------------------------------------
 void  GribRecord::setDataType(const zuchar t)
 {
@@ -179,26 +377,11 @@ std::string GribRecord::makeKey(int dataType,int levelType,int levelValue)
 	snprintf(ktmp, 32, "%d-%d-%d", dataType, levelType, levelValue);
 	return std::string(ktmp);
 }
-//-----------------------------------------
-GribRecord::~GribRecord()
-{
-    if (data) {
-        delete [] data;
-        data = NULL;
-    }
-    if (BMSbits) {
-        delete [] BMSbits;
-        BMSbits = NULL;
-    }
-    
-//if (dataType==GRB_TEMP) printf("record destroyed %s   %d\n", dataKey.c_str(), (int)curDate/3600);
-}
-
 //-------------------------------------------------------------------------------
 void  GribRecord::multiplyAllData(double k)
 {
-	for (zuint j=0; j<Nj; j++) {
-		for (zuint i=0; i<Ni; i++)
+	for (int j=0; j<Nj; j++) {
+		for (int i=0; i<Ni; i++)
 		{
 			if (hasValue(i,j)) {
 				data[j*Ni+i] *= k;
@@ -221,7 +404,7 @@ bool GribRecord::readGribSection0_IS(ZUFILE* file) {
 	while ( (zu_read(file, strgrib, 1) == 1)
 		   		&&  (strgrib[0] != 'G') )
 	{ }
-	
+
     if (strgrib[0] != 'G') {
         ok = false;
         eof = true;
@@ -247,14 +430,14 @@ bool GribRecord::readGribSection0_IS(ZUFILE* file) {
     totalSize = readInt3(file);
 
     editionNumber = readChar(file);
-    if (editionNumber != 1)  {
+    if (editionNumber != 1)  {	// 1=GRIB1, 2=GRIB2
         ok = false;
         eof = true;
         return false;
     }
 
     return true;
-}
+} 
 //----------------------------------------------
 // SECTION 1: THE PRODUCT DEFINITION SECTION (PDS)
 //----------------------------------------------
@@ -272,28 +455,26 @@ bool GribRecord::readGribSection1_PDS(ZUFILE* file) {
     idGrid   = data1[6];
     hasGDS = (data1[7]&128)!=0;
     hasBMS = (data1[7]&64)!=0;
-    
-    dataType = data1[8];	 // octet 9 = parameters and units
-	levelType = data1[9];
-	levelValue = makeInt2(data1[10],data1[11]);
+
+    dataType  = (int) data1[8];	 // octet 9 = parameters and units
+	levelType = (int) data1[9];
+	levelValue = (int) makeInt2(data1[10],data1[11]);
 
     refyear   = (data1[24]-1)*100+data1[12];
     refmonth  = data1[13];
     refday    = data1[14];
     refhour   = data1[15];
     refminute = data1[16];
-    
-    refDate = makeDate(refyear,refmonth,refday,refhour,refminute,0);
+
+    refDate = UTC_mktime(refyear,refmonth,refday,refhour,refminute,0);
 	sprintf(strRefDate, "%04d-%02d-%02d %02d:%02d", refyear,refmonth,refday,refhour,refminute);
 
     periodP1  = data1[18];
     periodP2  = data1[19];
     timeRange = data1[20];
     periodsec = periodSeconds(data1[17],data1[18],data1[19],timeRange);
-    curDate = makeDate(refyear,refmonth,refday,refhour,refminute,periodsec);
+    curDate = UTC_mktime(refyear,refmonth,refday,refhour,refminute,periodsec);
 
-//if (dataType == GRB_PRECIP_TOT) printf("P1=%d p2=%d\n", periodP1,periodP2);    
-    
     int decim;
     decim = (int)(((((zuint)data1[26]&0x7F)<<8)+(zuint)data1[27])&0x7FFF);
     if (data1[26]&0x80)
@@ -332,21 +513,23 @@ bool GribRecord::readGribSection2_GDS(ZUFILE* file) {
 
     Ni  = readInt2(file);				// byte 7-8
     Nj  = readInt2(file);				// byte 9-10
-    La1 = readSignedInt3(file)/1000.0;	// byte 11-12-13
-    Lo1 = readSignedInt3(file)/1000.0;	// byte 14-15-16
+    ymin = readSignedInt3(file)/1000.0;	// byte 11-12-13
+    xmin = readSignedInt3(file)/1000.0;	// byte 14-15-16
     resolFlags = readChar(file);		// byte 17
-    La2 = readSignedInt3(file)/1000.0;	// byte 18-19-20
-    Lo2 = readSignedInt3(file)/1000.0;	// byte 21-22-23
-
-    if (Lo1>=0 && Lo1<=180 && Lo2<0) {
-        Lo2 += 360.0;    // cross the 180 deg meridien,beetwen alaska and russia
-    }
+    ymax = readSignedInt3(file)/1000.0;	// byte 18-19-20
+    xmax = readSignedInt3(file)/1000.0;	// byte 21-22-23
+	savXmin = xmin;
+	savXmax = xmax;
+	savYmin = ymin;
+	savYmax = ymax;
 
     Di  = readSignedInt2(file)/1000.0;	// byte 24-25
     Dj  = readSignedInt2(file)/1000.0;	// byte 26-27
-    
-    while ( Lo1> Lo2   &&  Di >0) {   // horizontal size > 360 °
-        Lo1 -= 360.0;
+	savDi = Di;
+	savDj = Dj;
+	
+    while ( xmin> xmax   &&  Di >0) {   // horizontal size > 360 °
+        xmin -= 360.0;
     }
     hasDiDj = (resolFlags&0x80) !=0;
     isEarthSpheric = (resolFlags&0x40) ==0;
@@ -356,41 +539,19 @@ bool GribRecord::readGribSection2_GDS(ZUFILE* file) {
     isScanIpositive = (scanFlags&0x80) ==0;
     isScanJpositive = (scanFlags&0x40) !=0;
     isAdjacentI     = (scanFlags&0x20) ==0;
-    
-   	if (Lo2 > Lo1) {
-	    lonMin = Lo1;
-	    lonMax = Lo2;
-	}
-  	else {
-	    lonMin = Lo2;
-	    lonMax = Lo1;
-	}
-   	if (La2 > La1) {
-	    latMin = La1;
-	    latMax = La2;
-	}
-  	else {
-	    latMin = La2;
-	    latMax = La1;
-	}    
+	
+	
 	if (Ni<=1 || Nj<=1) {
 		erreur("Record %d: Ni=%d Nj=%d",id,Ni,Nj);
 		ok = false;
 	}
-	else {
-		Di = (Lo2-Lo1) / (Ni-1);
-		Dj = (La2-La1) / (Nj-1);
+	else { 
+		Di = (xmax-xmin) / (Ni-1);
+		Dj = (ymax-ymin) / (Nj-1);
 	}
 
-if (false) {
-printf("====\n");
-printf("Lo1=%f Lo2=%f    La1=%f La2=%f\n", Lo1,Lo2,La1,La2);
-printf("Ni=%d Nj=%d\n", Ni,Nj);
-printf("hasDiDj=%d Di,Dj=(%f %f)\n", hasDiDj, Di,Dj);
-printf("hasBMS=%d\n", hasBMS);
-printf("isScanIpositive=%d isScanJpositive=%d isAdjacentI=%d\n",
-                        isScanIpositive,isScanJpositive,isAdjacentI );
-}
+//print ("readGribSection2_GDS");	
+
     return ok;
 }
 //----------------------------------------------
@@ -425,7 +586,7 @@ bool GribRecord::readGribSection3_BMS(ZUFILE* file) {
 bool GribRecord::readGribSection4_BDS(ZUFILE* file) {
     fileOffset4  = zu_tell(file);
     sectionSize4 = readInt3(file);  // byte 1-2-3
-    
+
     zuchar flags  = readChar(file);			// byte 4
     scaleFactorE = readSignedInt2(file);	// byte 5-6
     refValue     = readFloat4(file);		// byte 7-8-9-10
@@ -450,7 +611,7 @@ bool GribRecord::readGribSection4_BDS(ZUFILE* file) {
         erreur("Record %d: need double values",id);
         ok = false;
     }
-    
+
     if (!ok) {
         return ok;
     }
@@ -461,7 +622,7 @@ bool GribRecord::readGribSection4_BDS(ZUFILE* file) {
         erreur("Record %d: out of memory",id);
         ok = false;
     }
-    
+
     zuint  startbit  = 0;
     int  datasize = sectionSize4-11;
     zuchar *buf = new zuchar[datasize+4];  // +4 pour simplifier les décalages ds readPackedBits
@@ -479,7 +640,7 @@ bool GribRecord::readGribSection4_BDS(ZUFILE* file) {
     }
 
     // Read data in the order given by isAdjacentI
-    zuint i, j, x;
+    int i, j, x;
     int ind;
     if (isAdjacentI) {
         for (j=0; j<Nj; j++) {
@@ -490,7 +651,7 @@ bool GribRecord::readGribSection4_BDS(ZUFILE* file) {
                 else {
                     ind = j*Ni+i;
                 }
-                if (hasValue(i,j)) {
+                if (hasValueInBitBMS(i,j)) {
                     x = readPackedBits(buf, startbit, nbBitsInPack);
                     data[ind] = (refValue + x*scaleFactorEpow2)/decimalFactorD;
                     startbit += nbBitsInPack;
@@ -510,7 +671,7 @@ bool GribRecord::readGribSection4_BDS(ZUFILE* file) {
                 else {
                     ind = j*Ni+i;
                 }
-                if (hasValue(i,j)) {
+                if (hasValueInBitBMS(i,j)) {
                     x = readPackedBits(buf, startbit, nbBitsInPack);
                     startbit += nbBitsInPack;
                     data[ind] = (refValue + x*scaleFactorEpow2)/decimalFactorD;
@@ -646,7 +807,7 @@ zuint GribRecord::readPackedBits(zuchar *buf, zuint first, zuint nbBits)
 {
     zuint oct = first / 8;
     zuint bit = first % 8;
-    
+
     zuint val = (buf[oct]<<24) + (buf[oct+1]<<16) + (buf[oct+2]<<8) + (buf[oct+3]);
     val = val << bit;
     val = val >> (32-nbBits);
@@ -657,9 +818,7 @@ zuint GribRecord::readPackedBits(zuchar *buf, zuint first, zuint nbBits)
 void  GribRecord::setRecordCurrentDate (time_t t)
 {
 	curDate = t;
-    
     struct tm *date = gmtime(&t);
-    
     zuint year   = date->tm_year+1900;
     zuint month  = date->tm_mon+1;
 	zuint day    = date->tm_mday;
@@ -667,30 +826,6 @@ void  GribRecord::setRecordCurrentDate (time_t t)
 	zuint minute = date->tm_min;
 	sprintf(strCurDate, "%04d-%02d-%02d %02d:%02d", year,month,day,hour,minute);
 }
-
-//----------------------------------------------
-time_t GribRecord::makeDate(
-            zuint year,zuint month,zuint day,zuint hour,zuint min,zuint sec) {
-    struct tm date;
-    date.tm_sec  = sec;         /* seconds */
-    date.tm_min  = min;         /* minutes */
-    date.tm_hour = hour;        /* hours */
-    date.tm_mday = day;         /* day of the month */
-    date.tm_mon  = month-1;     /* month */
-    date.tm_year = year-1900;   /* year */
-    date.tm_wday   = 0;         /* day of the week */
-    date.tm_yday   = 0;         /* day in the year */
-    date.tm_isdst  = 0;         /* daylight saving time */
-	
-	time_t   temps = -1;
-	char sdate[64];
-	sprintf(sdate, "%04d-%02d-%02d 00:00:00", year,month,day);
-    QDateTime dt = QDateTime::fromString(sdate, "yyyy-MM-dd HH:mm:ss");
-    dt.setTimeSpec(Qt::UTC);
-	dt = dt.addSecs (hour*3600+min*60+sec);
-	temps = dt.toTime_t();
-	return temps;
-}
 //----------------------------------------------
 zuint GribRecord::periodSeconds(zuchar unit,zuchar P1,zuchar P2,zuchar range) {
     zuint res, dur;
@@ -744,9 +879,9 @@ zuint GribRecord::periodSeconds(zuchar unit,zuchar P1,zuchar P2,zuchar range) {
 
 
 //===============================================================================================
-double GribRecord::getInterpolatedValue(double px, double py, bool numericalInterpolation) const
+double GribRecord::getInterpolatedValue (double px, double py, bool interpolate) const
 {
-    double val;
+    double val; 
     if (!ok || Di==0 || Dj==0) {
         return GRIB_NOTDEF;
     }
@@ -758,16 +893,28 @@ double GribRecord::getInterpolatedValue(double px, double py, bool numericalInte
                 return GRIB_NOTDEF;
             }
         }
-    }
+    } 
     double pi, pj;     // coord. in grid unit
-    pi = (px-Lo1)/Di;
-    pj = (py-La1)/Dj;
-
+    pi = (px-xmin)/Di;
+    pj = (py-ymin)/Dj;
     // 00 10      point is in a square
     // 01 11
-    int i0 = (int) pi;  // point 00
-    int j0 = (int) pj;
-    
+    int i0 = (int) floor(pi);  // point 00
+    int j0 = (int) floor(pj);
+
+	// value very close to a grid point ?
+	double eps = 1e-4;
+	double ddx = fabs (pi-i0);
+	double ddy = fabs (pj-j0);
+	int ii = (ddx<eps) ? i0 : ((1-ddx)<eps) ? i0+1 : -1;
+	int jj = (ddy<eps) ? j0 : ((1-ddy)<eps) ? j0+1 : -1;
+	if (ii>=0 && jj>=0) {
+		if (hasValue(ii,jj))
+			return getValue (ii, jj);
+		else
+			return GRIB_NOTDEF;
+	}
+
     bool h00,h01,h10,h11;
     int nbval = 0;     // how many values in grid ?
     if ((h00=hasValue(i0,   j0)))
@@ -787,7 +934,7 @@ double GribRecord::getInterpolatedValue(double px, double py, bool numericalInte
     double dx = pi-i0;
     double dy = pj-j0;
 
-	if (! numericalInterpolation)
+	if (! interpolate)
 	{
 		if (dx < 0.5) {
 			if (dy < 0.5)
@@ -803,10 +950,11 @@ double GribRecord::getInterpolatedValue(double px, double py, bool numericalInte
 		}
 		return val;
 	}
-	    
+
     dx = (3.0 - 2.0*dx)*dx*dx;   // pseudo hermite interpolation
     dy = (3.0 - 2.0*dy)*dy*dy;
-    
+
+
     double xa, xb, xc, kx, ky;
     // Triangle :
     //   xa  xb
@@ -814,7 +962,7 @@ double GribRecord::getInterpolatedValue(double px, double py, bool numericalInte
     // kx = distance(xa,x)
     // ky = distance(xa,y)
     if (nbval == 4)
-    {
+    { 
         double x00 = getValue(i0,   j0);
         double x01 = getValue(i0,   j0+1);
         double x10 = getValue(i0+1, j0);
@@ -824,7 +972,7 @@ double GribRecord::getInterpolatedValue(double px, double py, bool numericalInte
         val =  (1.0-dy)*x1 + dy*x2;
         return val;
     }
-    else {    
+    else {
         // here nbval==3, check the corner without data
         if (!h00) {
             //printf("! h00  %f %f\n", dx,dy);
@@ -876,8 +1024,25 @@ double GribRecord::getInterpolatedValue(double px, double py, bool numericalInte
     }
     return val;
 }
-
-
+//--------------------------------------------------------------------------
+double GribRecord::getValueOnRegularGrid (DataCode dtc, int i, int j ) const
+{
+	if ( getDataCode() != dtc )
+		return GRIB_NOTDEF;
+	else
+		return getValue (i,j);
+}
+//--------------------------------------------------------------------------
+double  GribRecord::getInterpolatedValue (
+						DataCode dtc,
+						double px, double py,
+						bool interpolate) const 
+{
+	if ( getDataCode() != dtc )
+		return GRIB_NOTDEF;
+	else
+		return getInterpolatedValueUsingRegularGrid (dtc,px,py,interpolate);
+}
 
 
 
