@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,20 +25,20 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QPainter>
 
 #include "MapDrawer.h"
-#include "GribPlot.h"
-#include "IacPlot.h"
 #include "LonLatGrid.h"
+#include "Font.h"
+#include "DataQString.h"
 
 //===========================================================
 MapDrawer::MapDrawer(GshhsReader *gshhsReader)
 	: QObject()
-{    
+{
     imgEarth = NULL;
     imgAll   = NULL;
-    
+
     gisReader = new GisReader();
     gisReaderIsNew = true;
-    
+
     assert(gisReader);
 	this->gshhsReader = gshhsReader;
 	initGraphicsParameters();
@@ -50,9 +50,20 @@ MapDrawer::MapDrawer(const MapDrawer &model)
     imgEarth = NULL;
     imgAll   = NULL;
 	gisReader = model.gisReader;
-    gisReaderIsNew = false;
+    gisReaderIsNew = false;		// don't delete pointer
 	this->gshhsReader = new GshhsReader(*model.gshhsReader);
-
+	
+	colorMapData = model.colorMapData;
+	colorMapSmooth = model.colorMapSmooth;
+	temperatureLabelsAlt = model.temperatureLabelsAlt;
+	windArrowsAltitude   = model.windArrowsAltitude;
+	geopotentialData = model.geopotentialData;
+	showGeopotential = model.showGeopotential;
+	showGeopotentialLabels = model.showGeopotentialLabels;
+	geopotentialStep = model.geopotentialStep;
+	geopotentialMin = model.geopotentialMin;
+	geopotentialMax = model.geopotentialMax;
+	
 	initGraphicsParameters();
 	updateGraphicsParameters();
 }
@@ -62,32 +73,38 @@ MapDrawer::~MapDrawer()
 	if (gisReaderIsNew) {
 		delete gisReader;
 	}
-}	
-    
+}
+
 //===========================================================
 void MapDrawer::initGraphicsParameters()
-{    
-    showCountriesBorders  = Util::getSetting("showCountriesBorders", true).toBool();
-    showRivers   = Util::getSetting("showRivers", false).toBool();
+{
     showCitiesNamesLevel = Util::getSetting("showCitiesNamesLevel", 0).toInt();
     showCountriesNames = Util::getSetting("showCountriesNames", false).toBool();
+    showCountriesBorders  = Util::getSetting("showCountriesBorders", true).toBool();
+    showRivers   = Util::getSetting("showRivers", false).toBool();
     showLonLatGrid   = Util::getSetting("showLonLatGrid", true).toBool();
-    
+
     showIsobars  = Util::getSetting("showIsobars", true).toBool();
     showIsobarsLabels = Util::getSetting("showIsobarsLabels", false).toBool();
     isobarsStep = Util::getSetting("isobarsStep", 2).toDouble();
     showPressureMinMax = Util::getSetting("showPressureMinMax", false).toBool();
-    
+
     showIsotherms0  = Util::getSetting("showIsotherms0", false).toBool();
     showIsotherms0Labels  = Util::getSetting("showIsotherms0Labels", false).toBool();
     isotherms0Step = Util::getSetting("isotherms0Step", 50).toDouble();
-    
-    colorMapMode = Util::getSetting("colorMapMode", MapDrawer::drawWind).toInt();
-    
-    colorMapSmooth = Util::getSetting("colorMapSmooth", true).toBool();
+
+	QString  strdtc = DataCodeStr::serialize (DataCode(GRB_WIND_XY2D,LV_ABOV_GND,10));
+    strdtc = Util::getSetting("colorMapData", strdtc).toString();
+	colorMapData = DataCodeStr::unserialize (strdtc);
+
+	strdtc = DataCodeStr::serialize (DataCode(GRB_GEOPOT_HGT,LV_ISOBARIC,850));
+    strdtc = Util::getSetting("geopotentialLinesData", strdtc).toString();
+	setGeopotentialData (DataCodeStr::unserialize (strdtc));
+
+	colorMapSmooth = Util::getSetting("colorMapSmooth", true).toBool();
     showWindArrows  = Util::getSetting("showWindArrows", true).toBool();
     showBarbules = Util::getSetting("showBarbules", true).toBool();
-    
+
     showTemperatureLabels = Util::getSetting("showTemperatureLabels", false).toBool();
     showGribGrid = Util::getSetting("showGribGrid", false).toBool();
 
@@ -95,27 +112,62 @@ void MapDrawer::initGraphicsParameters()
 
 //-------------------------------------------
 void MapDrawer::updateGraphicsParameters()
-{            
+{
     backgroundColor  = Util::getSetting("backgroundColor", QColor(0,0,45)).value<QColor>();
     seaColor  = Util::getSetting("seaColor", QColor(50,50,150)).value<QColor>();
     landColor = Util::getSetting("landColor", QColor(200,200,120)).value<QColor>();
-    
+
     seaBordersPen.setColor(Util::getSetting("seaBordersLineColor", QColor(40,45,30)).value<QColor>());
     seaBordersPen.setWidthF(Util::getSetting("seaBordersLineWidth", 1.8).toDouble());
-    
+
     boundariesPen.setColor(Util::getSetting("boundariesLineColor", QColor(40,40,40)).value<QColor>());
     boundariesPen.setWidthF(Util::getSetting("boundariesLineWidth", 1.4).toDouble());
-    
+
     riversPen.setColor(Util::getSetting("riversLineColor", QColor(50,50,150)).value<QColor>());
     riversPen.setWidthF(Util::getSetting("riversLineWidth", 1.0).toDouble());
-    
+
     isobarsPen.setColor(Util::getSetting("isobarsLineColor", QColor(80,80,80)).value<QColor>());
     isobarsPen.setWidthF(Util::getSetting("isobarsLineWidth", 2.0).toDouble());
-    
+
     isotherms0Pen.setColor(Util::getSetting("isotherms0LineColor", QColor(200,120,100)).value<QColor>());
     isotherms0Pen.setWidthF(Util::getSetting("isotherms0LineWidth", 1.6).toDouble());
 }
 
+//---------------------------------------------------------------------
+void MapDrawer::setGeopotentialData (const DataCode &dtc)
+{
+	if (dtc.getAltitude().levelType == LV_ISOBARIC) {
+		geopotentialData = dtc;
+		switch (dtc.getAltitude().levelValue) {
+			case 850:
+				geopotentialMin = 1000;
+				geopotentialMax = 2000;
+				break;
+			case 700:
+				geopotentialMin = 2000;
+				geopotentialMax = 4000;
+				break;
+			case 500:
+				geopotentialMin = 4000;
+				geopotentialMax = 7000;
+				break;
+			case 300:
+				geopotentialMin = 7000;
+				geopotentialMax = 11000;
+				break;
+			case 200:
+				geopotentialMin = 8000;
+				geopotentialMax = 16000;
+				break;
+		}
+		geopotentialStep = Util::getSetting("drawGeopotentialLinesStep", 10).toInt();
+	}
+	else {
+		geopotentialMin = 0;
+		geopotentialMax = 0;
+	}
+}
+
 //---------------------------------------------------------------------
 void MapDrawer::draw_Map_Background(bool isEarthMapValid, Projection *proj)
 {
@@ -125,7 +177,7 @@ void MapDrawer::draw_Map_Background(bool isEarthMapValid, Projection *proj)
 	}
 	imgAll = new QPixmap(proj->getW(), proj->getH());
 	assert(imgAll);
-	
+
 	QPainter pnt(imgAll);
 	pnt.setRenderHint(QPainter::Antialiasing, true);
 
@@ -151,7 +203,7 @@ void MapDrawer::draw_Map_Background(bool isEarthMapValid, Projection *proj)
 }
 //----------------------------------------------------------------------
 void MapDrawer::draw_Map_Foreground(QPainter &pnt, Projection *proj)
-{		
+{
 	if (gshhsReader != NULL)
 	{
 		pnt.setPen(seaBordersPen);
@@ -176,7 +228,7 @@ void MapDrawer::draw_Map_Foreground(QPainter &pnt, Projection *proj)
 	if (showCitiesNamesLevel > 0) {
 		gisReader->drawCitiesNames(pnt, proj, showCitiesNamesLevel);
 	}
-}		
+}
 
 //=======================================================================
 // GSHHS Map
@@ -207,7 +259,7 @@ void MapDrawer::draw_GSHHS (
 //=======================================================================
 // IAC Files
 //=======================================================================
-void MapDrawer::draw_GSHHSandIAC (
+void MapDrawer::draw_GSHHS_and_IAC (
 			QPainter &pntGlobal,
 			bool mustRedraw, bool isEarthMapValid,
 			Projection *proj,
@@ -230,7 +282,7 @@ void MapDrawer::draw_GSHHSandIAC (
 		// Dessin des données Meteo
 		//===================================================
 		draw_MeteoData_IAC (pnt, proj, iacPlot);
-		
+
 		//===================================================
 		// Cartouche
 		//===================================================
@@ -240,13 +292,13 @@ void MapDrawer::draw_GSHHSandIAC (
     pntGlobal.drawPixmap(0,0, *imgAll);
 }
 //=======================================================================
-// GRIB1 Files
+// Gridded data
 //=======================================================================
-void MapDrawer::draw_GSHHSandGRIB1 (
+void MapDrawer::draw_GSHHS_and_GriddedData (
 			QPainter &pntGlobal,
 			bool mustRedraw, bool isEarthMapValid,
 			Projection *proj,
-			GribPlot   *gribPlot
+			GriddedPlotter   *plotter
 	)
 {
     if (mustRedraw  ||  !isEarthMapValid)
@@ -254,15 +306,15 @@ void MapDrawer::draw_GSHHSandGRIB1 (
 		//===================================================
 		// Dessin du fond de carte
 		//===================================================
-		draw_Map_Background(isEarthMapValid, proj);
-		
+		draw_Map_Background (isEarthMapValid, proj);
+
 		//===================================================
 		// Dessin des données Meteo
 		//===================================================
-		QPainter pnt(imgAll);
-		pnt.setRenderHint(QPainter::Antialiasing, true);
-		draw_MeteoData_GRIB1 (pnt, proj, gribPlot);
-		
+		QPainter pnt (imgAll);
+		pnt.setRenderHint (QPainter::Antialiasing, true);
+		draw_MeteoData_Gridded (pnt, proj, plotter);
+
 		//===================================================
 		// Dessin des bordures et frontières
 		//===================================================
@@ -271,12 +323,277 @@ void MapDrawer::draw_GSHHSandGRIB1 (
 		//===================================================
 		// Cartouche
 		//===================================================
-		draw_Cartouche_GRIB1(pnt, proj, gribPlot);
+		draw_Cartouche_Gridded (pnt, proj, plotter);
     }
     // Recopie l'image complète
     pntGlobal.drawPixmap(0,0, *imgAll);
 }
 
+//===================================================================
+// Draw gridded data
+//===================================================================
+void MapDrawer::draw_MeteoData_Gridded 
+			( QPainter &pnt, Projection *proj,
+			GriddedPlotter   *plotter )
+{
+	plotter->draw_CoveredZone (pnt, proj);
+	
+	Altitude mapAltitude =  colorMapData.getAltitude ();
+	
+	if (showWindArrows) {
+		hasWindForArrows = true;
+		if (plotter->hasData (DataCode(GRB_WIND_XY2D, mapAltitude))) {
+			windArrowsAltitude = mapAltitude;
+		}
+		else if (plotter->hasData (DataCode(GRB_WIND_XY2D,LV_ABOV_GND,10))) {
+			windArrowsAltitude = Altitude (LV_ABOV_GND,10);
+		}
+		else if (plotter->hasData (DataCode(GRB_WIND_XY2D,LV_ABOV_GND,3))) {
+			windArrowsAltitude = Altitude (LV_ABOV_GND,3);
+		}
+		else if (plotter->hasData (DataCode(GRB_WIND_XY2D,LV_ABOV_GND,2))) {
+			windArrowsAltitude = Altitude (LV_ABOV_GND,2);
+		}
+		else if (plotter->hasData (DataCode(GRB_WIND_XY2D,LV_ABOV_GND,1))) {
+			windArrowsAltitude = Altitude (LV_ABOV_GND,1);
+		}
+		else if (plotter->hasData (DataCode(GRB_WIND_XY2D,LV_GND_SURF,0))) {
+			windArrowsAltitude = Altitude (LV_GND_SURF,0);
+		}
+		else {
+			hasWindForArrows = false;
+		}
+	}
+	
+	temperatureLabelsAlt = Altitude (LV_ABOV_GND,2);
+	switch (colorMapData.dataType) 
+	{
+		case GRB_WIND_XY2D :
+			windArrowsColor.setRgb(255, 255, 255);
+			break;
+		case GRB_TEMP :
+		case GRB_TEMP_POT :
+		case GRB_DEWPOINT :
+			temperatureLabelsAlt = mapAltitude;	
+			windArrowsColor.setRgb(255, 255, 255);
+			break;
+		case GRB_DIFF_TEMPDEW :
+		case GRB_CLOUD_TOT :
+		case GRB_HUMID_REL :
+			windArrowsColor.setRgb(180, 180, 80);
+			break;
+		case GRB_CAPE :
+			windArrowsColor.setRgb(100, 80, 80);
+			break;
+		case GRB_PRECIP_TOT :
+		case GRB_SNOW_DEPTH :
+		case GRB_SNOW_CATEG :
+		case GRB_FRZRAIN_CATEG :
+			windArrowsColor.setRgb(140, 120, 100);
+			break;
+		default:
+			windArrowsColor.setRgb(25, 25, 25);
+			break;
+	}
+	
+	//-------------------------------------------------------
+	// draw complete colored map
+	//-------------------------------------------------------
+	plotter->draw_ColoredMapPlain (colorMapData, colorMapSmooth,pnt,proj);
+	//-------------------------------------------------------
+
+	std::list<IsoLine *> listIsobars;
+	std::list<IsoLine *> listIsotherms0;
+	std::list<IsoLine *> listGeopotential;
+	
+	if (! plotter->hasData (GRB_PRESSURE,LV_MSL,0))
+		showIsobars = false;
+	if (! plotter->hasData (GRB_GEOPOT_HGT,LV_ISOTHERM0,0))
+		showIsotherms0 = false;
+	if (! plotter->hasData (geopotentialData))
+		showGeopotential = false;
+
+	if (showIsobars) {
+		pnt.setPen (isobarsPen);
+		plotter->complete_listIsolines (&listIsobars,
+						   DataCode (GRB_PRESSURE,LV_MSL,0),
+						   84000, 112000, isobarsStep*100);
+        plotter->draw_listIsolines (listIsobars, pnt,proj);
+	}
+
+	if (showIsotherms0) {
+		pnt.setPen (isotherms0Pen);
+		plotter->complete_listIsolines (&listIsotherms0,
+						   DataCode (GRB_GEOPOT_HGT,LV_ISOTHERM0,0),
+						   0, 15000, isotherms0Step );
+        plotter->draw_listIsolines (listIsotherms0, pnt,proj);
+	}
+
+	if (showGeopotential) {
+		pnt.setPen (geopotentialsPen);
+		plotter->complete_listIsolines (&listGeopotential,
+						   geopotentialData,
+						   geopotentialMin, geopotentialMax, geopotentialStep );
+        plotter->draw_listIsolines (listGeopotential, pnt,proj);
+	}
+
+	if (showWindArrows && hasWindForArrows) {
+		plotter->draw_WIND_Arrows (windArrowsAltitude, showBarbules, windArrowsColor, pnt, proj);
+	}
+
+	if (showIsobarsLabels && showIsobars) {
+		QColor color (40,40,40);
+        plotter->draw_listIsolines_labels (listIsobars, 0.01, color, pnt,proj);
+	}
+	if (showIsotherms0Labels && showIsotherms0) {
+		QColor color(200,80,80);
+		double coef = Util::getDataCoef (DataCode (GRB_GEOPOT_HGT,LV_ISOTHERM0,0));
+        plotter->draw_listIsolines_labels (listIsotherms0, coef, color, pnt,proj);
+	}
+	if (showGeopotentialLabels && showGeopotential) {
+		QColor color(200,80,80);
+		double coef = Util::getDataCoef (DataCode (GRB_GEOPOT_HGT,LV_ISOBARIC,0));
+        plotter->draw_listIsolines_labels (listGeopotential, coef, color, pnt,proj);
+	}
+
+	if (showPressureMinMax) {
+		plotter->draw_DATA_MinMax ( 
+						DataCode(GRB_PRESSURE,LV_MSL,0),
+						101200,
+						"L", "H",
+						Font::getFont(FONT_GRIB_PressHL),
+						QColor(0,0,0),
+						pnt, proj);
+	}
+	if (showTemperatureLabels) {
+		plotter->draw_DATA_Labels (
+					DataCode(GRB_TEMP,temperatureLabelsAlt),  
+					Font::getFont(FONT_GRIB_Temp),
+					QColor(0,0,0),
+					Util::formatTemperature_short,  
+					pnt, proj);
+	}
+
+	//===================================================
+	// Grille
+	//===================================================
+	if (showGribGrid) {
+		pnt.setPen(QColor (80,80,80));
+		plotter->draw_GridPoints (pnt, proj);
+	}
+}
+//-------------------------------------------------------------
+// Cartouche : dates de la prévision courante + infos générales
+//-------------------------------------------------------------
+void MapDrawer::draw_Cartouche_Gridded 
+		(QPainter &pnt, const Projection *proj, GriddedPlotter *plotter)
+{
+	GriddedReader *reader = plotter->getReader();
+    if (reader == NULL) {
+        return;
+    }
+
+    if (reader->getNumberOfDates() > 0)
+    {
+        QString  tref = "Ref: "+Util::formatDateTimeLong (reader->getRefDate());
+        QString  tval = "Val: "+Util::formatDateTimeLong (plotter->getCurrentDate());
+		QFont fontbig   = Font::getFont(FONT_MapInfo_Big);
+		QFont fontsmall = Font::getFont(FONT_MapInfo_Medium);
+		QFont fontsign  = Font::getFont(FONT_MapInfo_Small);
+		QFontMetrics fmbig   (fontbig);
+		QFontMetrics fmsmall (fontsmall);
+		QFontMetrics fmsign  (fontsign);
+        int n = 1;
+		
+        QStringList datalist;
+		if (colorMapData.dataType != GRB_TYPE_NOT_DEFINED)
+			datalist.append (DataCodeStr::toString_levelShort (colorMapData));
+		
+		if (showIsobars)
+			datalist.append (tr("Isobars MSL (hPa)"));
+		if (showIsotherms0)
+			datalist.append (tr("Isotherms 0°C") 
+				+" (" 
+				+Util::getDataUnit(DataCode(GRB_GEOPOT_HGT,LV_ISOTHERM0,0)) +")");
+		if (showTemperatureLabels)
+			datalist.append (tr("Temperature")
+					+" ("+AltitudeStr::toStringShort(temperatureLabelsAlt)+")");
+		if (showGeopotential)
+			datalist.append (tr("Geopotential")
+					+" "+AltitudeStr::toStringShort(geopotentialData.getAltitude())
+					+" (" 
+					+Util::getDataUnit(DataCode(GRB_GEOPOT_HGT,LV_ISOBARIC,0)) 
+					+")");
+		if (showWindArrows && hasWindForArrows)
+			datalist.append (tr("Wind arrows")
+					+" ("+AltitudeStr::toStringShort(windArrowsAltitude)+")");
+		
+        int dy, w, h, x, y, w1, w2;
+		QColor   transpcolor(255,255,255,180);
+		QColor   textcolor(20,20,20,255);
+
+		// use a gradient, because it's a bug sometimes with solid pattern (black background)
+		QLinearGradient gradient;
+		gradient.setColorAt(0, transpcolor);
+		gradient.setColorAt(1, transpcolor);
+		pnt.setBrush(gradient);
+		//---------------------------------------------------------
+		// forecast validity date (top left corner) + data type
+        QString  tval2 = Util::formatDateTimeLong(plotter->getCurrentDate());
+		n +=  datalist.size();
+		w1 = fmbig.width(tval2)+5;
+		w2 = 0;
+		// get largest string data
+		for (int i = 0; i < datalist.size(); i++) {
+        	int t = fmsmall.width (datalist.at(i))+5;
+			if (t>w2)
+				w2 = t;
+		}
+        w  = (w1>w2) ? w1 : w2;
+		dy = fontbig.pointSize()+1;
+        h = n*dy+8;
+        x = 0;
+        y = 0;
+        pnt.setPen(transpcolor);
+        pnt.drawRect(x,y,w,h);
+        pnt.setPen(textcolor);
+        pnt.setFont(fontbig);
+        pnt.drawText(x+3, y+dy+2, tval2);// forecast validity date
+		// show other data drawed in map
+        pnt.setFont(fontsmall);
+		for (int i = 0; i < datalist.size(); i++) {
+        	pnt.drawText(x+3, y+(i+2)*dy+4, datalist.at(i));
+		}
+		//--------------------------------------------------
+        // Date de référence (bas gauche)
+        pnt.setFont(fontsmall);
+		w = fmsmall.width(tref)+5;
+		dy = fontsmall.pointSize()+1;
+        h = dy+2;
+        x = 0;
+        y = proj->getH() - h-1;
+        pnt.setPen(transpcolor);
+        pnt.drawRect(x,y,w,h);
+        pnt.setPen(textcolor);
+        pnt.drawText(x+3, y+dy, tref);
+		//--------------------------------------------------
+        // Signature zyGrib (coin bas droite)
+		QString sign = tr("zyGrib");
+        pnt.setFont(fontsign);
+		w = fmsign.width(sign)+5;
+		dy = fontsign.pointSize()+1;
+        h = dy+2;
+        x = proj->getW() - w;
+        y = proj->getH() - h;
+        pnt.setPen(transpcolor);
+        pnt.drawRect(x,y,w,h);
+        pnt.setPen(textcolor);
+        pnt.drawText(x+3, y+dy, sign);
+	}
+}
+
+
+
 //===================================================================
 // Draw IAC data
 //===================================================================
@@ -287,7 +604,7 @@ bool iac_showPressureTroughLine = true;
 bool iac_showIsobars = true;
 bool iac_showIsobarsLabels = true;
 bool iac_showFronts = true;
-    
+
 
 	iacPlot->show_CoverZone(pnt, proj);
 
@@ -321,119 +638,22 @@ bool iac_showFronts = true;
 	}
 }
 
-//===================================================================
-// Draw GRIB data
-//===================================================================
-void MapDrawer::draw_MeteoData_GRIB1(QPainter &pnt, Projection *proj, GribPlot *gribPlot)
-{
-	gribPlot->show_CoverZone(pnt, proj);
-	
-	//QTime t1 = QTime::currentTime();
-	switch (colorMapMode)
-	{
-		case MapDrawer::drawWind :
-			windArrowsColor.setRgb(255, 255, 255);
-			gribPlot->draw_WIND_Color(pnt, proj, colorMapSmooth);
-			break;
-		case MapDrawer::drawRain :
-			windArrowsColor.setRgb(140, 120, 100);
-			gribPlot->draw_RAIN_Color(pnt, proj, colorMapSmooth);
-			break;
-		case MapDrawer::drawCloud :
-			windArrowsColor.setRgb(180, 180, 80);
-			gribPlot->draw_CLOUD_Color(pnt, proj, colorMapSmooth);
-			break;
-		case MapDrawer::drawHumid :
-			windArrowsColor.setRgb(180, 180, 80);
-			gribPlot->draw_HUMID_Color(pnt, proj, colorMapSmooth);
-			break;
-		case MapDrawer::drawTemp :
-			windArrowsColor.setRgb(255, 255, 255);
-			gribPlot->draw_Temp_Color(pnt, proj, colorMapSmooth);
-			break;
-		case MapDrawer::drawTempPot :
-			windArrowsColor.setRgb(255, 255, 255);
-			gribPlot->draw_TempPot_Color(pnt, proj, colorMapSmooth);
-			break;
-		case MapDrawer::drawDewpoint :
-			windArrowsColor.setRgb(255, 255, 255);
-			gribPlot->draw_Dewpoint_Color(pnt, proj, colorMapSmooth);
-			break;
-		case MapDrawer::drawDeltaDewpoint :
-			windArrowsColor.setRgb(180, 180, 80);
-			gribPlot->draw_DeltaDewpoint_Color(pnt, proj, colorMapSmooth);
-			break;
-		case MapDrawer::drawSnowDepth :
-			windArrowsColor.setRgb(140, 120, 100);
-			gribPlot->draw_SNOW_DEPTH_Color(pnt, proj, colorMapSmooth);
-			break;
-		case MapDrawer::drawSnowCateg :
-			windArrowsColor.setRgb(140, 120, 100);
-			gribPlot->draw_SNOW_CATEG_Color(pnt, proj, colorMapSmooth);
-			break;
-		case MapDrawer::drawFrzRainCateg :
-			windArrowsColor.setRgb(140, 120, 100);
-			gribPlot->draw_FRZRAIN_CATEG_Color(pnt, proj, colorMapSmooth);
-			break;
-		case MapDrawer::drawCAPEsfc :
-			windArrowsColor.setRgb(100, 80, 80);
-			gribPlot->draw_CAPEsfc(pnt, proj, colorMapSmooth);
-			break;
-	}
-	//printf("time show ColorMap = %d ms\n", t1.elapsed());
-
-	if (showIsobars) {
-		pnt.setPen(isobarsPen);
-		gribPlot->draw_Isobars(pnt, proj);
-	}
-
-	if (showIsotherms0) {
-		pnt.setPen(isotherms0Pen);
-		gribPlot->draw_Isotherms0(pnt, proj);
-	}
-
-	if (showWindArrows) {
-		gribPlot->draw_WIND_Arrows(pnt, proj, showBarbules, windArrowsColor);
-	}
-
-	if (showIsobarsLabels) {
-		gribPlot->draw_IsobarsLabels(pnt, proj);
-	}
-	if (showIsotherms0Labels) {
-		gribPlot->draw_Isotherms0Labels(pnt, proj);
-	}
-	if (showPressureMinMax) {
-		gribPlot->draw_PRESSURE_MinMax (pnt, proj);
-	}
-	if (showTemperatureLabels) {
-		gribPlot->draw_TEMPERATURE_Labels (pnt, proj);
-	}
-
-	//===================================================
-	// Grille GRIB
-	//===================================================
-	if (showGribGrid) {
-		gribPlot->draw_GribGrid(pnt, proj);
-	}
-}
 //-------------------------------------------------------------
 // Cartouche : dates de la prévision courante + infos générales
 void MapDrawer::draw_Cartouche_IAC(QPainter &pnt, const Projection *proj, IacPlot *iacPlot)
 {
-	IacReader *iacReader = iacPlot->getIacReader();
+	IacReader *iacReader = iacPlot->getReader();
 	if (iacReader!=NULL && iacReader->isOk())
 	{
-        int dy, w, h, x, y;
-		dy = 11;
-        QFont fontsmall("TypeWriter", dy-1, QFont::Bold, false);
-			fontsmall.setStyleHint(QFont::TypeWriter);
-			fontsmall.setStretch(QFont::Condensed);
-        QFont fontsign("TypeWriter", dy-1, QFont::Normal, false);
-			fontsign.setStyleHint(QFont::TypeWriter);
-			fontsign.setStretch(QFont::Condensed);
-        QFont fontbig("TypeWriter", dy+1, QFont::Bold, false);
-			fontbig.setStyleHint(QFont::TypeWriter);
-			fontbig.setStretch(QFont::Condensed);
+		QFont fontbig   = Font::getFont(FONT_MapInfo_Big);
+		QFont fontsmall = Font::getFont(FONT_MapInfo_Medium);
+		QFont fontsign  = Font::getFont(FONT_MapInfo_Small);
+		QFontMetrics fmbig   (fontbig);
+		QFontMetrics fmsmall (fontsmall);
+		QFontMetrics fmsign  (fontsign);
+		int dy, w, h, x, y, w1, w2;
+		dy = fontsmall.pointSize()+1;
+		
 		QColor   transpcolor(255,255,255,120);
 		QColor   textcolor(20,20,20,255);
         pnt.setBrush(transpcolor);
@@ -453,7 +673,7 @@ void MapDrawer::draw_Cartouche_IAC(QPainter &pnt, const Projection *proj, IacPlo
         }
         else if (type == IacReader::IAC_FORECAST)
         {
-			lig1 = tr("IAC fleetcode : Prévision");
+			lig1 = tr("IAC fleetcode : Forecast");
 			lig2 = lig2.sprintf("Ref: %02d-%02dh%02d UTC - Val:%+dh",
 						iacReader->getIacFileDay(),
 						iacReader->getIacFileHour(),
@@ -461,7 +681,9 @@ void MapDrawer::draw_Cartouche_IAC(QPainter &pnt, const Projection *proj, IacPlo
 						iacReader->getIacFileValidHour() );
         }
         pnt.setFont(fontbig);
-        w = 210;
+		w1 = fmbig.width(lig1)+5;
+		w2 = fmbig.width(lig2)+5;
+        w  = (w1>w2) ? w1 : w2;
         h = n*dy+8;
         x = 0;
         y = 0;
@@ -475,133 +697,47 @@ void MapDrawer::draw_Cartouche_IAC(QPainter &pnt, const Projection *proj, IacPlo
 
 		//--------------------------------------------------
         // Signature zyGrib (coin bas droite)
+		lig1 = tr("zyGrib");
         pnt.setFont(fontsign);
-        w = 44;
+        w = fmsign.width(lig1)+5;
+		dy = fontsign.pointSize()+1;
         h = dy+2;
         x = proj->getW() - w;
         y = proj->getH() - h;
         pnt.setPen(transpcolor);
         pnt.drawRect(x,y,w,h);
         pnt.setPen(textcolor);
-        pnt.drawText(x+3, y+dy, tr("zyGrib"));
+        pnt.drawText(x+3, y+dy, lig1);
 	}
 }
-//-------------------------------------------------------------
-// Cartouche : dates de la prévision courante + infos générales
-void MapDrawer::draw_Cartouche_GRIB1(QPainter &pnt, const Projection *proj, GribPlot *gribPlot)
+
+
+//===========================================================
+QPixmap * MapDrawer::createPixmap_GriddedData ( 
+						time_t date, 
+						bool isEarthMapValid, 
+						GriddedPlotter *plotter,
+						Projection *proj,
+						QList<POI*> lspois )
 {
-	GribReader *gribReader = gribPlot->getGribReader();
-    if (gribReader == NULL) {
-        return;
-    }
-    
-    if (gribReader->getNumberOfDates() > 0)
-    {
-        QString  tref = "Ref: "+Util::formatDateTimeLong(gribReader->getRefDate());
-        QString  tval = "Val: "+Util::formatDateTimeLong(gribPlot->getCurrentDate());
-        int n = 1;
-        QString data="";
-		switch (colorMapMode) {
-			case MapDrawer::drawWind:
-				data = tr("Vent à 10 m");
-				break;
-			case MapDrawer::drawCloud :
-				data = tr("Couverture nuageuse");
-				break;
-			case MapDrawer::drawRain :
-				data = tr("Précipitations");
-				break;
-			case MapDrawer::drawHumid :
-				data = tr("Humidité relative");
-				break;
-			case MapDrawer::drawTemp :
-				data = tr("Température à 2 m");
-				break;
-			case MapDrawer::drawDewpoint :
-				data = tr("Point de rosée à 2 m");
-				break;
-			case MapDrawer::drawDeltaDewpoint :
-				data = tr("Ecart temp-pt de rosée");
-				break;
-		 
-			case MapDrawer::drawTempMin :
-				data = tr("Temp. Min à 2 m");
-				break;
-			case MapDrawer::drawTempMax :
-				data = tr("Temp. Max à 2 m");
-				break;
-			case MapDrawer::drawTempPot :
-				data = tr("Temp. Potentielle");
-				break;
-		 
-			case MapDrawer::drawSnowDepth :
-				data = tr("Neige (épaisseur)");
-				break;
-			case MapDrawer::drawSnowCateg :
-				data = tr("Neige (chute possible)");
-				break;
-			case MapDrawer::drawCAPEsfc :
-				data = tr("CAPE (surface)");
-				break;
-		}
-		if (data != "") {
-			n ++;
-		}
-        int dy, w, h, x, y;
-		
-		dy = 11;
-        QFont fontsmall("TypeWriter", dy-1, QFont::Bold, false);
-			fontsmall.setStyleHint(QFont::TypeWriter);
-			fontsmall.setStretch(QFont::Condensed);
-        QFont fontsign("TypeWriter", dy-1, QFont::Normal, false);
-			fontsign.setStyleHint(QFont::TypeWriter);
-			fontsign.setStretch(QFont::Condensed);
-        QFont fontbig("TypeWriter", dy+1, QFont::Bold, false);
-			fontbig.setStyleHint(QFont::TypeWriter);
-			fontbig.setStretch(QFont::Condensed);
-		QColor   transpcolor(255,255,255,120);
-		QColor   textcolor(20,20,20,255);
-        pnt.setBrush(transpcolor);
-		//---------------------------------------------------------
-		// forecast validity date (top left corner) + data type
-        QString  tval2 = Util::formatDateTimeLong(gribPlot->getCurrentDate());
-        pnt.setFont(fontbig);
-        w = 190;
-        h = n*dy+8;
-        x = 0;
-        y = 0;
-        pnt.setPen(transpcolor);
-        pnt.drawRect(x,y,w,h);
-        pnt.setPen(textcolor);
-        pnt.drawText(x+3, y+dy+2, tval2);// forecast validity date
-		if (data != "") {
-        	pnt.drawText(x+3, y+2*dy+7, data);	// type of color map
-		}
-		//--------------------------------------------------
-        // Date de référence (bas gauche)
-        pnt.setFont(fontsmall);
-        w = 190;
-        h = dy+2;
-        x = 0;
-        y = proj->getH() - h;
-        pnt.setPen(transpcolor);
-        pnt.drawRect(x,y,w,h);
-        pnt.setPen(textcolor);
-        pnt.drawText(x+3, y+dy, tref);
-		//--------------------------------------------------
-        // Signature zyGrib (coin bas droite)
-        pnt.setFont(fontsign);
-        w = 44;
-        h = dy+2;
-        x = proj->getW() - w;
-        y = proj->getH() - h;
-        pnt.setPen(transpcolor);
-        pnt.drawRect(x,y,w,h);
-        pnt.setPen(textcolor);
-        pnt.drawText(x+3, y+dy, tr("zyGrib"));
+	QPixmap *pixmap = new QPixmap(proj->getW(), proj->getH());
+	if (pixmap == NULL) {
+		return NULL;
 	}
+	QPainter pnt;
+	pnt.begin (pixmap);
+		plotter->setCurrentDate (date);
+		
+		this->draw_GSHHS_and_GriddedData (pnt, true, isEarthMapValid, proj, plotter);
+
+		// Ajoute les pOIs visibles
+		for (int i=0; i<lspois.size(); i++) {
+			POI *poi = lspois.at(i);
+			if (poi->isVisible()) {
+				poi->drawContent (pnt, proj, true);
+			}
+		}
+	pnt.end();
+	return pixmap;
 }
 
-
-
-
