@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -26,13 +26,16 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "Terrain.h"
 #include "Orthodromie.h"
+#include "Font.h"
+#include "DataQString.h"
 
 //---------------------------------------------------------
 // Constructeur
 //---------------------------------------------------------
-Terrain::Terrain(QWidget *parent, Projection *proj_, GshhsReader *gshhsReader)
+Terrain::Terrain (QWidget *parent, Projection *proj_, GshhsReader *gshhsReader)
     : QWidget(parent)
 {
+	setSizePolicy (QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
     quality = 0;
     selX0 = selY0 = 0;
     selX1 = selY1 = 0;
@@ -52,7 +55,7 @@ Terrain::Terrain(QWidget *parent, Projection *proj_, GshhsReader *gshhsReader)
 	//---------------------------------------------------
 	drawer = new MapDrawer(gshhsReader);
 	assert(drawer);
-	currentMeteoFileType = NOFILE_TYPE;
+	currentFileType = DATATYPE_NONE;
     
     //----------------------------------------------------------------------------
     showOrthodromie   = Util::getSetting("showOrthodromie", false).toBool();
@@ -64,24 +67,22 @@ Terrain::Terrain(QWidget *parent, Projection *proj_, GshhsReader *gshhsReader)
     mustRedraw = true;
 
     proj = proj_;
-    
-    gribPlot = new GribPlot(interpolateValues, windArrowsOnGribGrid);
-    assert(gribPlot);
-    
-    iacPlot = new IacPlot();
-    assert(iacPlot);
 	
+    //---------------------------------------------------------------
+	griddedPlot = new GribPlot ();
+	assert(griddedPlot);
+
+	iacPlot = new IacPlot();
+    assert(iacPlot);
+
+    //---------------------------------------------------------------
 	updateGraphicsParameters();
     setIsobarsStep(drawer->isobarsStep);
     
     isSelectionZoneEnCours = false;
-/*    setPalette(QPalette(backgroundColor));
-    setAutoFillBackground(true);*/
     
     setMouseTracking(true);
     setFocusPolicy(Qt::StrongFocus);
-
-    
 	
 	//---------------------------------------------------
 	// Define new cross cursor, more visible
@@ -126,6 +127,7 @@ void Terrain::updateGraphicsParameters()
     drawer->updateGraphicsParameters();
     int v = 180;
     selectColor     = QColor(v,v,v);
+	griddedPlot->setCloudsColorMode("cloudsColorMode");
     
 	isEarthMapValid = false;
 	mustRedraw = true;
@@ -287,7 +289,7 @@ void Terrain::setCitiesNamesLevel  (int level) {
 }
 
 //-------------------------------------------------------
-void Terrain::setMapQuality(int q) {
+void Terrain::setMapQuality (int q) {
     indicateWaitingMap();
     if (quality != q) {
         if (drawer->gshhsReader == NULL)
@@ -306,23 +308,22 @@ void Terrain::setMapQuality(int q) {
 }
 
 //-------------------------------------------------------
-void Terrain::setColorMapMode(int mode )
+void Terrain::setColorMapData (const DataCode &dtc)
 {
-    if (drawer->colorMapMode != mode)
+    if (drawer->getColorMapData() != dtc)
     {
-		Util::setSetting("colorMapMode", mode);
-        drawer->colorMapMode = mode;
+		Util::setSetting ("colorMapData", DataCodeStr::serialize(dtc));
+        drawer->setColorMapData (dtc);
         mustRedraw = true;
         update();
     }
 }
-
 //-------------------------------------------------------
 void Terrain::setDuplicateFirstCumulativeRecord (bool b) {
     if (duplicateFirstCumulativeRecord != b) {
         duplicateFirstCumulativeRecord = b;
         Util::setSetting("duplicateFirstCumulativeRecord", b);
-	    gribPlot->duplicateFirstCumulativeRecord( b );
+	    griddedPlot->duplicateFirstCumulativeRecord (b);
         mustRedraw = true;
         update();
     }
@@ -332,7 +333,7 @@ void Terrain::setInterpolateValues (bool b) {
     if (interpolateValues != b) {
         interpolateValues = b;
         Util::setSetting("interpolateValues", b);
-	    gribPlot->interpolateValues( b );
+	    griddedPlot->setInterpolateValues (b);
         mustRedraw = true;
         update();
     }
@@ -342,7 +343,7 @@ void Terrain::setWindArrowsOnGribGrid (bool b) {
     if (windArrowsOnGribGrid != b) {
         windArrowsOnGribGrid = b;
         Util::setSetting("windArrowsOnGribGrid", b);
-	    gribPlot->windArrowsOnGribGrid( b );
+	    griddedPlot->setWindArrowsOnGrid (b);
         mustRedraw = true;
         update();
     }
@@ -405,8 +406,8 @@ void Terrain::setDrawIsobars(bool b) {
 //-------------------------------------------------------
 void Terrain::setIsobarsStep(double step)
 {
-    if (gribPlot != NULL) {
-        gribPlot->setIsobarsStep(step);
+    if (griddedPlot != NULL) {
+        griddedPlot->setIsobarsStep(step);
         Util::setSetting("isobarsStep", step);
         drawer->isobarsStep = step;
         mustRedraw = true;
@@ -435,8 +436,8 @@ void Terrain::setDrawIsotherms0(bool b) {
 //-------------------------------------------------------
 void Terrain::setIsotherms0Step(double step)
 {
-    if (gribPlot != NULL) {
-        gribPlot->setIsotherms0Step(step);
+    if (griddedPlot != NULL) {
+        griddedPlot->setIsotherms0Step(step);
         Util::setSetting("isotherms0Step", step);
         drawer->isotherms0Step = step;
         mustRedraw = true;
@@ -492,98 +493,136 @@ bool  Terrain::getSelectedLine(double *x0, double *y0, double *x1, double *y1)
 //---------------------------------------------------------
 bool  Terrain::getGribFileRectangle(double *x0, double *y0, double *x1, double *y1)
 {
-	if (gribPlot) {
-		if (gribPlot->isGribReaderOk()) {
-			return gribPlot->getGribReader()->getZoneExtension (x0,y0, x1,y1);
+	if (griddedPlot) {
+		if (griddedPlot->isReaderOk()) {
+			return griddedPlot->getReader()->getZoneExtension (x0,y0, x1,y1);
 		}
 	}
     return false;
 }
 
 //---------------------------------------------------------
-// Grib or Iac files
+// Grib or IAC files or ...
 //---------------------------------------------------------
-void Terrain::loadIacFile(QString fileName, bool zoom)
+FileDataType Terrain::loadMeteoDataFile (QString fileName, bool zoom)
 {
-	currentMeteoFileType = IAC_TYPE;
     indicateWaitingMap();
-    iacPlot->loadIacFile( fileName );
-    isSelectionZoneEnCours = false;
+	currentFileType = DATATYPE_NONE;
+	bool ok = false;
+	if (! ok) {		// try to load a GRIB file
+		//DBG("try to load a GRIB file");
+		if (griddedPlot != NULL)
+			delete griddedPlot;
+		griddedPlot = new GribPlot ();
+		assert(griddedPlot);
+		griddedPlot->loadFile( fileName );    // GRIB file ?
+		if (griddedPlot->isReaderOk()) {
+			currentFileType = DATATYPE_GRIB;
+			ok = true;
+		}
+	}
+	if (! ok) {		// try to load a MBLUE file
+		//DBG("try to load a MBLUE file");
+		if (griddedPlot != NULL)
+			delete griddedPlot;
+		griddedPlot = new MbluePlot ();
+		assert(griddedPlot);
+		griddedPlot->loadFile( fileName );    // GRIB file ?
+		if (griddedPlot->isReaderOk()) {
+			currentFileType = DATATYPE_MBLUE;
+			ok = true;
+		}
+	}
+	if (! ok) {		// try to load a IAC file
+		//DBG("try to load a IAC file");
+		iacPlot->loadFile( fileName );      // IAC file ?
+		if (iacPlot->isReaderOk()) {
+			currentFileType = DATATYPE_IAC;
+			ok = true;
+		}
+	}
+
+	if (ok) {	// initializes data plotter
+		switch (currentFileType) {
+			case DATATYPE_GRIB :
+				griddedPlot->setInterpolateValues (interpolateValues);
+				griddedPlot->setWindArrowsOnGrid  (windArrowsOnGribGrid);
+				griddedPlot->duplicateFirstCumulativeRecord (duplicateFirstCumulativeRecord);
+				griddedPlot->setCurrentDateClosestFromNow ();
+				break;
+			case DATATYPE_MBLUE :
+				griddedPlot->setInterpolateValues (interpolateValues);
+				griddedPlot->setWindArrowsOnGrid  (windArrowsOnGribGrid);
+				griddedPlot->setCurrentDateClosestFromNow ();
+				break;
+			default :
+				break;
+		}
+	}
+	else {
+		DBG("ERROR: unknown file type");
+	}
+	
+	isSelectionZoneEnCours = false;
     selX0 = selY0 = 0;
     selX1 = selY1 = 0;
     isEarthMapValid = false;
 	mustRedraw = true;
     if (zoom) {
-        zoomOnIacFile();    // Zoom sur la zone couverte par le fichier GRIB
+        zoomOnFileZone();    // Zoom sur la zone couverte par le fichier GRIB
     }
     update();
-}
-//---------------------------------------------------------
-void Terrain::zoomOnIacFile()
-{
-//printf("Terrain::zoomOnGribFile\n");
-    IacReader * iacReader= iacPlot->getIacReader();
-    if (iacReader!=NULL && iacReader->isOk()) {
-        double x0,y0, x1,y1, mh, mv;
-        if (iacReader->getZoneExtension(&x0,&y0, &x1,&y1))
-        {
-            mh = fabs(x0-x1)*0.05;
-            mv = fabs(y0-y1)*0.05;
-            proj->setVisibleArea(x0-mh,y0-mv, x1+mh,y1+mv);
-            setProjection(proj);
-        }
-    }
+	return currentFileType;
 }
 
 //---------------------------------------------------------
-void Terrain::loadGribFile(QString fileName, bool zoom)
+GriddedPlotter *Terrain::getGriddedPlotter ()
 {
-	currentMeteoFileType = GRIB1_TYPE;
-    indicateWaitingMap();
-    
-    gribPlot->loadGribFile( fileName );
-    gribPlot->duplicateFirstCumulativeRecord( duplicateFirstCumulativeRecord );
-    
-    isSelectionZoneEnCours = false;
-    selX0 = selY0 = 0;
-    selX1 = selY1 = 0;
-    isEarthMapValid = false;
-	mustRedraw = true;
-    if (zoom) {
-        zoomOnGribFile();    // Zoom sur la zone couverte par le fichier GRIB
+    if (   currentFileType == DATATYPE_GRIB
+		|| currentFileType == DATATYPE_MBLUE
+	) {
+		return griddedPlot;
     }
+	else {
+		return NULL;
+	}
+}
+
+//---------------------------------------------------------
+void   Terrain::closeMeteoDataFile()
+{
+	currentFileType = DATATYPE_NONE;
+	mustRedraw = true;
     update();
 }
+
 //---------------------------------------------------------
-void Terrain::zoomOnGribFile()
+void Terrain::zoomOnFileZone()
 {
 	double x0,y0, x1,y1, mh, mv;
-    if (currentMeteoFileType == GRIB1_TYPE)
-    {
-		GribReader * gribReader= gribPlot->getGribReader();
-		if (gribReader != NULL) {
-			if (gribReader->getZoneExtension(&x0,&y0, &x1,&y1))
-			{
-				mh = fabs(x0-x1)*0.05;
-				mv = fabs(y0-y1)*0.05;
-				proj->setVisibleArea(x0-mh,y0-mv, x1+mh,y1+mv);
-				setProjection(proj);
-			}
+	bool ok = false;
+    if (   currentFileType == DATATYPE_GRIB
+		|| currentFileType == DATATYPE_MBLUE
+    ) {
+		if (griddedPlot!=NULL && griddedPlot->isReaderOk()) {
+			ok = griddedPlot->getReader()->getZoneExtension(&x0,&y0, &x1,&y1);
 		}
     }
-    else if (currentMeteoFileType == IAC_TYPE) {
-		IacReader * iacReader= iacPlot->getIacReader();
-		if (iacReader!=NULL && iacReader->isOk()) {
-			if (iacReader->getZoneExtension(&x0,&y0, &x1,&y1))
-			{
-				mh = fabs(x0-x1)*0.05;
-				mv = fabs(y0-y1)*0.05;
-				proj->setVisibleArea(x0-mh,y0-mv, x1+mh,y1+mv);
-				setProjection(proj);
-			}
+    else if (currentFileType == DATATYPE_IAC) {
+		if (iacPlot!=NULL && iacPlot->isReaderOk()) {
+			ok = iacPlot->getReader()->getZoneExtension(&x0,&y0, &x1,&y1);
 		}
     }
+
+	if (ok)
+	{
+		mh = fabs(x0-x1)*0.05;
+		mv = fabs(y0-y1)*0.05;
+		proj->setVisibleArea(x0-mh,y0-mv, x1+mh,y1+mv);
+		setProjection(proj);
+	}
 }
+
 //---------------------------------------------------------
 void Terrain::slotMustRedraw()
 {
@@ -595,10 +634,10 @@ void Terrain::slotMustRedraw()
 //---------------------------------------------------------
 void Terrain::setCurrentDate(time_t t)
 {
-    if (gribPlot->getCurrentDate() != t)
+    if (griddedPlot->getCurrentDate() != t)
     {
         indicateWaitingMap();
-        gribPlot->setCurrentDate(t);
+        griddedPlot->setCurrentDate(t);
 		mustRedraw = true;
         update();
     }
@@ -723,12 +762,10 @@ printf("clic: i=%d j=%d   x=%f y=%f\n", e->x(), e->y(), x0, y0);*/
 
 //---------------------------------------------------------
 void Terrain::mouseMoveEvent (QMouseEvent * e) {
-//printf("move %d %d\n",e->x(),e->y());
+//DBG ("move %d %d",e->x(),e->y());
     if (isSelectionZoneEnCours)
     {
         proj->screen2map(e->x(),e->y(), &selX1, &selY1);
-/*        selX1 = ceil (selX1*12)/12;
-        selY1 = floor(selY1*12)/12;*/
         update();
     }
     emit mouseMoved(e);
@@ -801,7 +838,7 @@ void Terrain::slot_Zoom_Sel()
         isSelectionZoneEnCours = false;
         isEarthMapValid = false;
 		mustRedraw = true;
-        zoomOnGribFile();
+        zoomOnFileZone();
         update();
     }
 	setProjection(proj);
@@ -849,6 +886,49 @@ void Terrain::setShowPOIs(bool show)
 	}
 }
 
+//-------------------------------------------------------
+void Terrain::setGeopotentialData (const DataCode &dtc)
+{
+//	griddedPlot->getReader()->hasData (dtc);
+    if (drawer->getGeopotentialData() != dtc) {
+		Util::setSetting ("geopotentialLinesData", DataCodeStr::serialize(dtc));
+        drawer->setGeopotentialData (dtc);
+        mustRedraw = true;
+        update();
+    }
+}
+//-------------------------------------------------------
+void Terrain::setDrawGeopotential (bool b)
+{
+    if (drawer->showGeopotential != b) {
+		drawer->showGeopotential = b;
+		Util::setSetting ("drawGeopotentialLines", b);
+		mustRedraw = true;
+		update();
+    }
+}
+//-------------------------------------------------------
+void Terrain::setDrawGeopotentialLabels (bool b)
+{
+    if (drawer->showGeopotentialLabels != b) {
+		drawer->showGeopotentialLabels = b;
+		Util::setSetting ("drawGeopotentialLinesLabels", b);
+		mustRedraw = true;
+		update(); 
+    }
+}
+//-------------------------------------------------------
+void Terrain::setGeopotentialStep (int step)
+{
+    if (drawer->geopotentialStep != step)  {
+		drawer->geopotentialStep = step;
+		Util::setSetting ("drawGeopotentialLinesStep", step);
+		mustRedraw = true;
+		update();
+    }
+}
+
+
 //---------------------------------------------------------
 // paintEvent
 //---------------------------------------------------------
@@ -863,15 +943,21 @@ void Terrain::paintEvent(QPaintEvent * /*event*/)
         QCursor oldcursor = cursor();
         setCursor(Qt::WaitCursor);
         
-        if (currentMeteoFileType == GRIB1_TYPE) {
-        	drawer->draw_GSHHSandGRIB1 (pnt, mustRedraw, isEarthMapValid, proj, gribPlot);
+        switch (currentFileType) {
+			case DATATYPE_GRIB :
+			case DATATYPE_MBLUE :
+				drawer->draw_GSHHS_and_GriddedData 
+						(pnt, mustRedraw, isEarthMapValid, proj, 
+						 griddedPlot
+						);
+				break;
+			case DATATYPE_IAC :
+				drawer->draw_GSHHS_and_IAC 
+						(pnt, mustRedraw, isEarthMapValid, proj, iacPlot);
+				break;
+			default :
+				drawer->draw_GSHHS (pnt, mustRedraw, isEarthMapValid, proj);
         }
-        else if (currentMeteoFileType == IAC_TYPE) {
-        	drawer->draw_GSHHSandIAC (pnt, mustRedraw, isEarthMapValid, proj, iacPlot);
-        }
-		else {
-        	drawer->draw_GSHHS (pnt, mustRedraw, isEarthMapValid, proj);
-		}
 		
         setCursor(oldcursor);
 		isEarthMapValid = true;
@@ -901,30 +987,80 @@ void Terrain::paintEvent(QPaintEvent * /*event*/)
     
     if (pleaseWait) {
         // Write the message "please wait..." on the map
-        QFont fontWait("Helvetica", 12, QFont::Bold, true);
+        QFont fontWait = Font::getFont(FONT_MapWait);
         QFontMetrics fmet(fontWait);
         pnt.setPen(QColor(Qt::white));
         r = 80;
         transp = QColor(r,r,r, 80);
         pnt.setFont(fontWait);
         pnt.setBrush(transp);
-        QString txt = tr("  Calculs en cours...  ");
+        QString txt = tr("  Please wait...  ");
         QRect rect = fmet.boundingRect(txt);
 
         rect.moveTo(20,20);
         pnt.drawRect(rect);
         pnt.drawText(rect, Qt::AlignHCenter|Qt::AlignVCenter , txt);
     }
-
-// 	bool showCenter = true;
-//     if (showCenter) {
-//     	int s = 20;
-//         pnt.setPen(QColor(Qt::red));
-// 		pnt.drawLine(width()/2-s,height()/2, width()/2+s,height()/2);
-// 		pnt.drawLine(width()/2,height()/2-s, width()/2,height()/2+s);
-//     }
-
+}
+//------------------------------------------------------------------
+time_t Terrain::getCurrentDate()
+{
+	switch (currentFileType) {
+		case DATATYPE_GRIB :
+		case DATATYPE_MBLUE :
+			if (griddedPlot && griddedPlot->isReaderOk() )
+				return griddedPlot->getCurrentDate();
+			break;
+		case DATATYPE_IAC :
+			DBGS("TODO: get date IAC")
+			return 0;
+			break;
+		default :
+			return 0;
+	}
+	return 0;
+}
+//------------------------------------------------------------------------
+QPixmap * Terrain::createPixmap (time_t date, int width, int height)
+{
+	Projection *scaledproj = proj->clone(); 
+	MapDrawer  *scaleddrawer = new MapDrawer (*drawer);
+	
+	if (scaledproj && scaleddrawer)
+	{
+		double x0,y0, x1,y1;
+		scaledproj->getVisibleArea (&x0,&y0, &x1,&y1);
+		scaledproj->setScreenSize  (width, height);
+		scaledproj->setVisibleArea (x0,y0, x1,y1);
+		switch (currentFileType) {
+			case DATATYPE_GRIB :
+			case DATATYPE_MBLUE :
+				if (griddedPlot && griddedPlot->isReaderOk() )
+				{
+					QPixmap *pixmap = 
+						scaleddrawer->createPixmap_GriddedData ( 
+										date, 
+										false, 
+										griddedPlot, 
+										scaledproj, 
+										getListPOIs() );
+					return pixmap;
+				}
+				break;
+			case DATATYPE_IAC :
+				if (iacPlot && iacPlot->isReaderOk() )
+				{
+					DBGS("TODO: save image IAC")
+				}
+				break;
+			default :
+					DBGS("TODO: save image without data")
+				return NULL;
+		}
+		delete scaledproj;
+		delete scaleddrawer;
+	}
+	return NULL;
 }
 
 
-
